#!BPY
"""Registration info for Blender menus:
Name: 'Blendigo v1.0.8 (.igs)'
Blender: 245
Group: 'Export'
Tooltip: 'material editor and exporter for Indigo v1.0.8'
"""
#
# ***** BEGIN GPL LICENSE BLOCK *****
#
# --------------------------------------------------------------------------
# BLENDIGO v1.0.8
# --------------------------------------------------------------------------
#
# Authors:
# * Indigo exporter - Nick Chapman, U3dreal, Zuegs, Ewout Fernhout, Leope, psor, Wedge, SmartDen, BIGFAN(BF), DaveC
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software Foundation,
# Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#
# ***** END GPL LICENCE BLOCK *****
# --------------------------------------------------------------------------


# Done ;) 
# --------------------------------------------
# done - add possibility to remove texture
# done - inverse absorption colors
# done - add "external" mat-type where you can specify an external indigo-XML include-file
# done - reworked "export mesh" "separate mesh file" and "separate mat file"
# done - switch for "save IGI"
# done - uniform sss (by SmartDen)
# done - remove light surface compensation - gain is equal to XML gain - use "efficacy" feature instead
# done - fixed "null" material export when used in blend-material
# done - indigo primitives (sphere, plane)
# done - linear tonemapping
# done - reworked f-stop and aperture-radius formulas
# done - fixed misstyped matExponentPathCB (by manuel)
# done - absorption color inversion on GUI removed (by SmartDen)
# done - max. turbidity to 20
# done - added exponent for linear tonemapping
# done - fix slowing down of GUI while working on big meshes
# done - dupliverts and duplifaces
# done - fixed animation export
# done - replaced ImageSelector with FileSelector for better performance
# done - fixed bug when object is visible on more than on layer
# done - fixed relative path routine when it uses an other drive
# done - add copy/paste functions
# done - background color picker
# done - filename popup for preview scene
# done - compatibility to indigo v0.7t7 (by SmartDen)
# done - set max value for phong exponent and glossy-transparent exponent to 1000000
# done - if preview scene file can't be found, then try to find old preview-scenes with .xml extension
# done - export dummy light material
# done - additional rendersize-options 150%, 200%
# done - filter/ignore objects without faces
# done - get primitive sphere diameter to fit inside mesh bounding-box
# done - added exponent for unform SSS value
# done - F-Stop, shutter and fim iso made into menus and added medium types for future #BF
# done - support for array modifier
# done - add world dimension factor
# done - added capability of different preview scenes by menu (by BF and DaveC)
# done - general tidy up of GUI and tooltips       #BF
# done - added super sample                        #BF
# done - all preview files use same material name to make new preview files easy to insert (By DaveC)
# done - updated preview scenes and added "IMPmatdb" preview scene
# done - added "&" to "launchIndigo" function to detach Indigo from Blender under Linux (by GNUdo)
# done - chnged Uniform SSS valur/exp range
# done - added presets menu - in readiness         #BF
# done - added common metals menu -in readiness    #BF
# done - added hybrid and threshold                #BF
# done - added glare                               #BF
# done - added cache KD-trees to disk              #BF
# done - removed strata and RR prob                #BF
# done - added dermis and epidermis to mediums     #BF
# done - added phase function to sss               #BF
# done - material conversion: converts blender material to indigo material if no indigo-material definition is found
# done - relative paths #SmartDen
# done - corrected the absorption color export
# done - added diffuse_transmitter material #SmartDen
# done - fixed blended diffuse_transitter #SmartDen
# done - added SSS translation #SmartDen


# To Do
#---------------------------------------------
# - right convertion of emitters
# - check with Nick for the file extension for include files
# - check auto exposure (can someone provide me the formulas to check this feature?)
# - check curve without extrude
# - edit box for filename replaces "default.igs"
# - fix dupli-verts when child-object is non-uniform scaled (perhaps bug in blenders-python-api)


######################################################
# Importing modules
######################################################

import math
import os
import sys
import copy
import Blender
import BPyMesh
from Blender import Mesh,Scene, Object, Material, Texture, Window
from Blender import Draw, BGL, Mathutils
from Blender.sys import *





########################################################################
########################################################################
########################################################################
### INDIGO material editor submodule - (by zuegs)
### ----------------------------------------------------------------

### Setup constants
evtMatBase = 100 # first event-number for material editor module

### Materialtype numbers
mLight = -1
mDiffuse = 0
mPhong = 1
mSpecular = 2
mBlend = 3
mExitPortal = 4
mInclude = 9

### Event-numbers
evtNoEvt = evtMatBase + 0
evtMatsMenu = evtMatBase + 1
evtMatLightIES = evtMatBase + 9
evtMatType = evtMatBase + 10
evtMatLightEfficacy = evtMatBase + 11
evtMatLightIESPath = evtMatBase + 12
evtMatCopy = evtMatBase + 14
evtMatPaste = evtMatBase + 15
evtMatRgbPicker = evtMatBase + 20
evtMatRgbPicker_s = evtMatBase + 16
evtMatBlendMatA = evtMatBase + 21
evtMatBlendMatB = evtMatBase + 22
evtMatAlbedoPath = evtMatBase + 23
evtMatBumpPath = evtMatBase + 24
evtMatExponentPath = evtMatBase + 25
evtMatBlendPath = evtMatBase + 26
evtMatSssRgbPicker = evtMatBase + 27
evtMatPreview = evtMatBase + 28
evtMatInclude = evtMatBase + 29
evtMatAbsPicker = evtMatBase + 30
evtPreviewScene = evtMatBase + 31     #BF 
evtMatConvert = evtMatBase + 33

### Globals
activeObject = None
activeMaterial = None
matsMenu = Draw.Create(0)
matClipboard = {}
matPreviewFile  = "IMPsphere.xml"     #BF


### GUI Globals
matGui = Draw.Create(0) #SmartDen
matType = 1   # -1:light 0:diffuse 1:phong 2:specular 3:Blend 9:XML
matTypeMenu = Draw.Create(1) #workaround for buggy GUI   
matDiffTrans = Draw.Create(0) #SmartDen
matLightType = Draw.Create(1) # 1:RGB 2:peak 3:blackbody 4:uniform
matLightGain = Draw.Create(1.0)
matLightBaseValue = Draw.Create(0.0)
matLightPeakMin = Draw.Create(300.0)
matLightPeakWidth = Draw.Create(400.0)
matLightTemp = Draw.Create(5000.0)
matLightEfficacyScale = Draw.Create(1)
matLightIES = Draw.Create(0)
matLightIESPath = Draw.Create('')
matLightPower = Draw.Create(100.0)
matLightEfficacy = Draw.Create(17.5)
matTogglePhong=Draw.Create(0)
matRgb = Draw.Create(0.8, 0.8, 0.8)
matRgbR = Draw.Create(0.8)
matRgbG = Draw.Create(0.8)
matRgbB = Draw.Create(0.8)
matRgb_s = Draw.Create(0.8, 0.8, 0.8)
matRgbR_s = Draw.Create(0.8)
matRgbG_s = Draw.Create(0.8)
matRgbB_s = Draw.Create(0.8)

matRgbGain = Draw.Create(1.0)
matIOR = Draw.Create(1.0)
matExponent = Draw.Create(100)
matNKData = Draw.Create('')
matSigma = Draw.Create(0.2)
matTransparent = Draw.Create(0)
matSpecularType = Draw.Create(1) # 1:specular 2:glossy_transparent
matPrecedence = Draw.Create(10)
matMediumType = Draw.Create(1) # 1:basic
matCauchyBCoeff = Draw.Create(0.0)
matAbsorptionType = Draw.Create(1) # 1:RGB
matAbs = Draw.Create(0.5, 0.5, 0.5)
matAbsR = Draw.Create(0.0)
matAbsG = Draw.Create(0.0)
matAbsB = Draw.Create(0.0)
matAbsGain = Draw.Create(1.0)
matSss = Draw.Create(0)
matSssType = Draw.Create(2) # 1:RGB
matSssR = Draw.Create(0.8)
matSssG = Draw.Create(0.8)
matSssB = Draw.Create(0.8)
matSssRgb = Draw.Create(0.8, 0.8, 0.8)
matSssGain = Draw.Create(1.0)
matSssUniform = Draw.Create(1.0)  # SmartDen
matSssUniformExp = Draw.Create(0.0)
matSssPhaseType = Draw.Create(1)      #BF
#matSssPhaseFunct = Draw.Create(0.5)   #BF
matSssPhaseSpec=Draw.Create(1)
matSssPhaseSpecUni = Draw.Create(0.5)
matSssPhaseSpecR=Draw.Create(0.5)
matSssPhaseSpecG=Draw.Create(0.5)
matSssPhaseSpecB=Draw.Create(0.5)
matAlbedoUVset = Draw.Create('uv')
matAlbedoPath = Draw.Create('')
matAlbedoExponent = Draw.Create(2.3)
matAlbedoA = Draw.Create(0.0)
matAlbedoB = Draw.Create(1.0)
matAlbedoC = Draw.Create(0.0)
matBumpUVset = Draw.Create('uv')
matBumpPath = Draw.Create('')
matBumpExponent = Draw.Create(1.0)
matBumpA = Draw.Create(0.0)
matBumpB = Draw.Create(0.01)
matBumpC = Draw.Create(0.0)
matExponentUVset = Draw.Create('uv')
matExponentPath = Draw.Create('')
matExponentExponent = Draw.Create(1.0)
matExponentA = Draw.Create(0.0)
matExponentB = Draw.Create(10000)
matExponentC = Draw.Create(1.0)
matBlendMatA = Draw.Create('')
matBlendMatB = Draw.Create('')
matBlendFactor = Draw.Create(0.5)
matBlendUVset = Draw.Create('uv')
matBlendPath = Draw.Create('')
matBlendExponent = Draw.Create(1.0)
matBlendA = Draw.Create(0.0)
matBlendB = Draw.Create(10000)
matBlendC = Draw.Create(1.0)
matInclude = Draw.Create('')
PreviewScene = Draw.Create(0)         #BF
#matsPreset = Draw.Create(0)           #BF
matCommonMetals = Draw.Create(1)      #BF
matHemoglobin = Draw.Create(0.001)    #BF
matMelanin = Draw.Create(0.15)        #BF
matEumelanin = Draw.Create(0.001)     #BF
matTurbidity=Draw.Create(2.2)
matMediumType = Draw.Create(1)        #BF

### ================================================================
### relpath(base, target) -> relative path
###		returns the relative path between base and target path
### todo: this propabely don't work for linux yet ;(
### ----------------------------------------------------------------
def relpath(base,target):
	base=Blender.sys.dirname(base)
	if target[0:2] == "\\\\" or target[0:2] == "//": #SmartDen
		return target[2:len(target)]
	if not os.path.isabs(base):
		base = os.path.abspath(base)
	if not os.path.isabs(target):
		target = os.path.abspath(target)
	if os.sep == "\\":
		base = os.path.normcase(base)
		target = os.path.normcase(target)
	if base == os.sep:
		return '.' + target
	baselist = base.split(os.sep)
	if baselist[-1] == "":
		baselist = baselist[:-1]
	targetlist = target.split(os.sep)
	i = 0
	top = min([len(baselist), len(targetlist)])
	while i < top and baselist[i] == targetlist[i]:
		i+=1
	if i == 0:
		return os.sep.join(targetlist)
	if i == len(baselist):
		return sep.join(targetlist[i:])
	else:
		return ('..' + os.sep) * (len(baselist) - i) + os.sep.join(targetlist[i:])

### ================================================================
### convertMaterial(material)
###		converts a blender-material-to an indigo-material
### ----------------------------------------------------------------
def convertMaterial(mat):
	d = {}
	transparent = (mat.mode & Material.Modes.RAYTRANSP) or (mat.getAlpha() < 1.0)
	#if transparent and (mat.mode & Material.Modes.RAYMIRROR) or mat.enableSSS:
	if transparent or mat.enableSSS:
	#if transparent and mat.getIOR() > 1: #SmartDen
		# SPECULAR
		d['type'] = 2
		d['precedence'] = 10
		d['cauchy_b_coeff'] = 0.0
		#if (mat.mode & Material.Modes.RAYMIRROR):
		d['specular.type'] = 1
		#else:
		#	d['specular.type'] = 2
		#	d['exponent'] = mat.hard*10
		if transparent:
			d['transparent'] = 1
			d['ior'] = mat.getIOR()
		else:
			d['transparent'] = 0
			d['ior'] = 1.0/(1.001-mat.getRayMirr())
			#d['ior'] = mat.getIOR() #SmartDen
		if d['ior'] < 1.01:  # as values equal or smaller than 1.0 gives unexpected results
			d['ior'] = 1.01
		d['abs.r'] = 1.0-mat.R  # absorption is the inverted color!
		d['abs.g'] = 1.0-mat.G
		d['abs.b'] = 1.0-mat.B
		#d['abs.gain'] = -1.4637 * math.log(1.0 - mat.getAlpha()*0.9999999)
		d['abs.gain'] = mat.getFilter()*100
		if mat.getFilter() == 0:
			d['abs.gain'] = 1
		if mat.enableSSS: #SmartDen
			d['sss'] = 1
			d['sss.r'] = mat.sssR
			d['sss.g'] = mat.sssG
			d['sss.b'] = mat.sssB
	elif transparent and (mat.getAlpha() == 0):
		d['type'] = 4
	elif (not transparent and mat.getIOR() > 1) or mat.getSpec() > 0: #SmartDen
		specfac = mat.getSpec()
		if specfac > 0.0001:
			# PHONG
			d['type'] = 1
			d['rgb.r'] = mat.R
			d['rgb.g'] = mat.G
			d['rgb.b'] = mat.B
			d['rgb.gain'] = mat.getRef()
			d['ior'] = 1.0+math.exp(mat.getSpec())/2.71
			#d['ior'] = mat.getIOR() #SmartDen
			d['exponent'] = mat.hard*10
		#else:
	elif mat.getIOR() == 1: #SmartDen
		# DIFFUSE
		if mat.getDiffuseShader() == Material.Shaders.DIFFUSE_ORENNAYAR:
			d['type'] = 5
			d['sigma'] = mat.getRoughness()
		#if mat.getDiffuseShader() == Material.Shaders.DIFFUSE_LAMBERT:
		else:
			d['type'] = 0
			if transparent:
				d['diffuse.transmitter'] = 1
		d['rgb.r'] = mat.R
		d['rgb.g'] = mat.G
		d['rgb.b'] = mat.B
		d['rgb.gain'] = mat.getRef()
	if mat.getEmit() > 0: #SmartDen
		d['type'] = -1
		d['matLightType'] = 1
		d['rgb.r'] = mat.R
		d['rgb.g'] = mat.G
		d['rgb.b'] = mat.B
	for mtex in mat.getTextures():
		if (mtex) and (mtex.texco & Blender.Texture.TexCo['UV']) and (mtex.tex) and (mtex.tex.type == Blender.Texture.Types.IMAGE) and (mtex.tex.image):
			if mtex.mapto & Blender.Texture.MapTo['COL']:
				d['albedo.path'] = mtex.tex.image.filename #SmartDen
				d['albedo.uv_set'] = 'uv'
				d['albedo.exponent'] = 2.3
				d['albedo.a'] = 0.0
				d['albedo.b'] = mtex.colfac
				d['albedo.c'] = 0.0
			if mtex.mapto & Blender.Texture.MapTo['NOR']:
				d['bump.path'] = mtex.tex.image.filename #SmartDen
				d['bump.uv_set'] = 'uv'
				d['bump.exponent'] = 1.0
				d['bump.a'] = 0.0
				d['bump.b'] = mtex.norfac * 0.01
				d['bump.c'] = 0.0
			# added by SmartDen
			if mtex.mapto & Blender.Texture.MapTo['SPEC']:
				d['exponent.path'] = mtex.tex.image.filename
				d['exponent.uv_set'] = 'uv'
				d['exponent.exponent'] = 1.0
				d['exponent.a'] = 0.0
				#d['exponent.b'] = mtex.norfac * 0.01
				d['exponent.b'] = 10000
				d['exponent.c'] = 0.0
			# SmartDen end	
	mat.properties['indigo'] = d
	drawGUI()


### ================================================================
### readWriteMaterial(material, write)
###		reads or writes material-data from or to blender material property
### ----------------------------------------------------------------
def readWriteValue(dict, name, draw, write, default):
	if write == 0:
		try:
			try:
				draw.val = dict[name]
			except KeyError:
				draw.val = default
		except ValueError:
			print "ValueError: on property \"%s\""%name
	else:
		dict[name] = draw.val


def readWriteMaterial(m, write=0):
	global matsMenu, activeMaterial, matType
	global matDiffTrans #SmartDen
	global matLightType, matLightGain, matLightBaseValue, matLightPeakMin, matLightPeakWidth, matLightTemp
	global matLightEfficacyScale, matLightIES, matLightIESPath, matLightPower, matLightEfficacy
	global matTogglePhong, matRgb, matRgbR, matRgbG, matRgbB,matRgb_s, matRgbR_s, matRgbG_s, matRgbB_s, matRgbGain
	global matSigma, matIOR, matTransparent, matExponent, matNKData, matCauchyBCoeff, matPrecedence
	global matSpecularType, matMediumType, matAbsorptionType, matSss, matSssType, matSssR, matSssG, matSssB, matSssGain, matSssUniform, matSssUniformExp, matSssPhaseType, matSssFuncSpec
	global matAlbedoUVset, matAlbedoPath, matAlbedoExponent, matAlbedoA, matAlbedoB, matAlbedoC
	global matBumpUVset, matBumpPath, matBumpExponent, matBumpA, matBumpB, matBumpC
	global matExponentUVset, matExponentPath, matExponentExponent, matExponentA, matExponentB, matExponentC
	global matBlendMatA, matBlendMatB, matBlendFactor, matBlendUVset, matBlendPath, matBlendExponent, matBlendA, matBlendB, matBlendC, PreviewScene, matPreviewFile
	#, matsPreset, 
	global matCommonMetals #BF
	global matHemoglobin, matTurbidity, matMelanin, matEumelanin, matAbsR, matAbsG, matAbsB, matAbsGain #BF
	if m != None:
		try:
			d = m.properties['indigo'].convert_to_pyobject()
		except KeyError:
			d = {}
			if write == 0:
				convertMaterial(m)
				d = m.properties['indigo'].convert_to_pyobject()
		if write == 0:
			try:
		 		matType = d['type']
			except KeyError:
				matType = 0
		else:
			d['type'] = matType
		readWriteValue(d, 'diffuse.transmitter', matDiffTrans, write, 0) #SmartDen
		readWriteValue(d, 'light.type', matLightType, write, 1)
		readWriteValue(d, 'light.gain', matLightGain, write, 1.0)
		readWriteValue(d, 'light.base', matLightBaseValue, write, 0.0)
		readWriteValue(d, 'light.peak_min', matLightPeakMin, write, 300.0)
		readWriteValue(d, 'light.peak_width', matLightPeakWidth, write, 400.0)
		readWriteValue(d, 'light.temperature', matLightTemp, write, 5000.0)
		readWriteValue(d, 'light.efficacy_scale', matLightEfficacyScale, write, 1)
		readWriteValue(d, 'light.ies', matLightIES, write, 0)
		readWriteValue(d, 'light.ies_path', matLightIESPath, write, '')
		readWriteValue(d, 'light.power_drawn', matLightPower, write, 100.0)
		readWriteValue(d, 'light.overall_luminous_efficacy', matLightEfficacy, write, 17.5)
		readWriteValue(d, 'toggle_phong', matTogglePhong, write, 0)
		readWriteValue(d, 'rgb.r', matRgbR, write, 0.8)
		readWriteValue(d, 'rgb.g', matRgbG, write, 0.8)
		readWriteValue(d, 'rgb.b', matRgbB, write, 0.8)
		readWriteValue(d, 'rgb.r_s', matRgbR_s, write, 0.8)
		readWriteValue(d, 'rgb.g_s', matRgbG_s, write, 0.8)
		readWriteValue(d, 'rgb.b_s', matRgbB_s, write, 0.8)
		readWriteValue(d, 'rgb.gain', matRgbGain, write, 1.0)
		readWriteValue(d, 'ior', matIOR, write, 1.0)
		readWriteValue(d, 'exponent', matExponent, write, 100)
		readWriteValue(d, 'nk_data', matNKData, write, '')
		readWriteValue(d, 'sigma', matSigma, write, 0.2)
		readWriteValue(d, 'transparent', matTransparent, write, 0)
		readWriteValue(d, 'specular.type', matSpecularType, write, 1)
		readWriteValue(d, 'precedence', matPrecedence, write, 10)
		readWriteValue(d, 'medium.type', matMediumType, write, 1)
		readWriteValue(d, 'hemoglobin', matHemoglobin, write, 0.001)      #BF
		readWriteValue(d, 'turbidity', matTurbidity, write, 2.2)
		readWriteValue(d, 'melanin', matMelanin, write, 0.15)             #BF
		readWriteValue(d, 'eumelanin', matEumelanin, write, 0.001)        #BF
		readWriteValue(d, 'cauchy_b_coeff', matCauchyBCoeff, write, 0.0)
		readWriteValue(d, 'absorption.type', matAbsorptionType, write, 1)
		readWriteValue(d, 'abs.r', matAbsR, write, 0.0)
		readWriteValue(d, 'abs.g', matAbsG, write, 0.0)
		readWriteValue(d, 'abs.b', matAbsB, write, 0.0)
		readWriteValue(d, 'abs.gain', matAbsGain, write, 1)
		readWriteValue(d, 'sss', matSss, write, 0)
		readWriteValue(d, 'sss.type', matSssType, write, 2)
		readWriteValue(d, 'sss.r', matSssR, write, 1)
		readWriteValue(d, 'sss.g', matSssG, write, 1)
		readWriteValue(d, 'sss.b', matSssB, write, 1)
		readWriteValue(d, 'sss.gain', matSssGain, write, 1.0)
		readWriteValue(d, 'sss.uniform', matSssUniform, write, 1.0)        # SmartDen
		readWriteValue(d, 'sss.uniformExp', matSssUniformExp, write, 0.0)
		#readWriteValue(d, 'sss.phasefunct' , matSssPhaseFunct, write, 0.5) #BF
		readWriteValue(d,'sss.phasespec',matSssPhaseSpec,write,1)
		readWriteValue(d,'sss.phasespec_uni',matSssPhaseSpecUni,write,0.5)
		readWriteValue(d,'sss.phasespec_r',matSssPhaseSpecR,write,0.5)
		readWriteValue(d,'sss.phasespec_g',matSssPhaseSpecG,write,0.5)
		readWriteValue(d,'sss.phasespec_b',matSssPhaseSpecB,write,0.5)
		readWriteValue(d, 'sss.phasetype' , matSssPhaseType, write, 1)     #BF
		readWriteValue(d, 'albedo.uv_set', matAlbedoUVset, write, 'uv')
		readWriteValue(d, 'albedo.path', matAlbedoPath, write, '')
		readWriteValue(d, 'albedo.exponent', matAlbedoExponent, write, 2.3)
		readWriteValue(d, 'albedo.a', matAlbedoA, write, 0.0)
		readWriteValue(d, 'albedo.b', matAlbedoB, write, 1.0)
		readWriteValue(d, 'albedo.c', matAlbedoC, write, 0.0)
		readWriteValue(d, 'bump.uv_set', matBumpUVset, write, 'uv')
		readWriteValue(d, 'bump.path', matBumpPath, write, '')
		readWriteValue(d, 'bump.exponent', matBumpExponent, write, 1.0)
		readWriteValue(d, 'bump.a', matBumpA, write, 0.0)
		readWriteValue(d, 'bump.b', matBumpB, write, 0.01)
		readWriteValue(d, 'bump.c', matBumpC, write, 0.0)
		readWriteValue(d, 'exponent.uv_set', matExponentUVset, write, 'uv')
		readWriteValue(d, 'exponent.path', matExponentPath, write, '')
		readWriteValue(d, 'exponent.exponent', matExponentExponent, write, 1.0)
		readWriteValue(d, 'exponent.a', matExponentA, write, 0.0)
		readWriteValue(d, 'exponent.b', matExponentB, write, 10000)
		readWriteValue(d, 'exponent.c', matExponentC, write, 1.0)
		readWriteValue(d, 'blend.a_name', matBlendMatA, write, '')
		readWriteValue(d, 'blend.b_name', matBlendMatB, write, '')
		readWriteValue(d, 'blend.factor', matBlendFactor, write, 0.5)
		readWriteValue(d, 'blend.uv_set', matBlendUVset, write, 'uv')
		readWriteValue(d, 'blend.path', matBlendPath, write, '')
		readWriteValue(d, 'blend.exponent', matBlendExponent, write, 1.0)
		readWriteValue(d, 'blend.a', matBlendA, write, 0.0)
		readWriteValue(d, 'blend.b', matBlendB, write, 1.0)
		readWriteValue(d, 'blend.c', matBlendC, write, 0.0)
		readWriteValue(d, 'include', matInclude, write, '')
		if write != 0:
			m.properties['indigo'] = d


### ================================================================
### exportMaterial(material) -> string
###		returns indigo XML code for material
### ----------------------------------------------------------------
def exportMaterial(mat, file, exportedmatlist, includelist=[], newmatname=''):
	def readValue(dict, name, default):
		try:
			r = dict[name]
		except KeyError:
			r = default
		return r
	#End of readValue
	print("Filename: %s"%file.name)
	if mat != None:
		try:
			d = mat.properties['indigo']
		except KeyError:
			d = {}
			convertMaterial(mat)
			d = mat.properties['indigo'].convert_to_pyobject()
			print 'WARNING: material "%s" has no Indigo material definitions - converting blender material'%(mat.getName())  
		matname = mat.getName()
		if newmatname != '':
			matname = newmatname
		exportedmatlist.append(matname)
		m = []
		matType = readValue(d, 'type', 0)
# SSS redone by BF for phase function and skin
		matMediumType = readValue(d, 'medium.type', 1)
		if (matType >= 0) and (matType != mInclude) and (matType != mExitPortal):
			if (matType == 2) and (matMediumType == 1):
				file.write("\t<medium>\n")
				file.write("\t\t<name>%s</name>\n"%(matname))
				file.write("\t\t<precedence>%g</precedence>\n"%(readValue(d, 'precedence', 1.0)))
				file.write("\t\t<basic>\n")
				file.write("\t\t\t<ior>%g</ior>\n"%(readValue(d, 'ior', 1.0)))
				file.write("\t\t\t<cauchy_b_coeff>%g</cauchy_b_coeff>\n"%(readValue(d, 'cauchy_b_coeff', 1.0)))
				file.write("\t\t\t<absorption_coefficient_spectrum>\n")
				if readValue(d, 'absorption.type', 1) == 1:
					gain = readValue(d, 'abs.gain', 1.0)
					file.write("\t\t\t\t<rgb>\n")
					file.write("\t\t\t\t\t<rgb>%g %g %g</rgb>\n\t\t\t\t\t<gamma>2.2</gamma>\n"%((readValue(d, 'abs.r', 0.0))*gain, (readValue(d, 'abs.g', 0.0))*gain, (readValue(d, 'abs.b', 0.0))*gain))
					file.write("\t\t\t\t</rgb>\n")
				else:
					file.write("\t\t\t\t<uniform>\n")
					file.write("\t\t\t\t\t<value>0</value>\n")
					file.write("\t\t\t\t</uniform>\n")
				file.write("\t\t\t</absorption_coefficient_spectrum>\n")
				if (readValue(d, 'sss', 0) != 0):
					file.write("\t\t\t<subsurface_scattering>\n")
					file.write("\t\t\t\t<scattering_coefficient_spectrum>\n")
					if (readValue(d, 'sss.type', 1) == 1):
						gain = readValue(d, 'sss.gain', 1.0)
						file.write("\t\t\t\t\t<rgb>\n")
						file.write("\t\t\t\t\t\t<rgb>%g %g %g</rgb>\n\t\t\t\t\t\t<gamma>2.2</gamma>\n"%(readValue(d, 'sss.r', 0.0)*gain, readValue(d, 'sss.g', 0.0)*gain, readValue(d, 'sss.b', 0.0)*gain))
						file.write("\t\t\t\t\t</rgb>\n")
					if (readValue(d,'sss.type',1) == 2):
						file.write("\t\t\t\t\t<uniform>\n")
						file.write("\t\t\t\t\t\t<value>%g</value>\n"%(readValue(d,'sss.uniform',0.0)*(10**readValue(d, 'sss.uniformExp',0.0))))
						file.write("\t\t\t\t\t</uniform>\n")
					file.write("\t\t\t\t</scattering_coefficient_spectrum>\n")
					if (readValue(d, 'sss.phasetype', 1) == 1):					
						file.write("\t\t\t\t<phase_function>\n")
						file.write("\t\t\t\t\t<uniform/>\n")
						file.write("\t\t\t\t</phase_function>\n")
					if (readValue(d, 'sss.phasetype', 1) == 2):
						file.write("\t\t\t\t<phase_function>\n")
						file.write("\t\t\t\t\t<henyey_greenstein>\n")
						file.write("\t\t\t\t\t\t<g_spectrum>\n")
						if (readValue(d,'sss.phasespec',1) == 1): 
							file.write("\t\t\t\t\t\t\t<uniform>\n")
							file.write("\t\t\t\t\t\t\t\t<value>%g</value>\n"%(readValue(d, 'sss.phasespec_uni', 0.5))) 
							file.write("\t\t\t\t\t\t\t</uniform>\n")
						else:
							file.write("\t\t\t\t\t\t\t<rgb>\n")
							file.write("\t\t\t\t\t\t\t\t<rgb>%g %g %g</rgb>\n"%(readValue(d,'sss.phasespec_r',0.5),readValue(d,'sss.phasespec_g',0.5),readValue(d,'sss.phasespec_b',0.5)))
							file.write("\t\t\t\t\t\t\t\t<gamma>2.2</gamma>\n")
							file.write("\t\t\t\t\t\t\t</rgb>\n")
						file.write("\t\t\t\t\t\t</g_spectrum>\n")
						file.write("\t\t\t\t\t</henyey_greenstein>\n")
						file.write("\t\t\t\t</phase_function>\n")
					file.write("\t\t\t</subsurface_scattering>\n")
				file.write("\t\t</basic>\n")
				file.write("\t</medium>\n")
			if (matType == 2) and (matMediumType == 2): #specular and dermis
				file.write("\t<medium\n>")
				file.write("\t\t<name>%s</name>\n"%(matname))
				file.write("\t\t<precedence>%g</precedence>\n"%(readValue(d, 'precedence', 1.0)))	
				file.write("\t\t<dermis>\n")
				file.write("\t\t\t<hemoglobin_fraction>%g</hemoglobin_fraction>\n"%(readValue(d, 'hemoglobin', 0.15)))	
				file.write("\t\t</dermis>\n")
				file.write("\t</medium>\n")
			if (matType == 2) and (matMediumType == 3): #specular and epidermis
				file.write("\t<medium>\n")
				file.write("\t\t<name>%s</name>\n"%(matname))
				file.write("\t\t<precedence>%g</precedence>"%(readValue(d, 'precedence', 1.0))) 	
				file.write("\t\t<epidermis>")
				file.write("\t\t\t<melanin_fraction>%g</melanin_fraction>"%(readValue(d, 'melanin', 0.001)))	
				file.write("\t\t\t<melanin_type_blend>%g</melanin_type_blend>"%(readValue(d, 'eumelanin', 0.001)))	
				file.write("\t\t</epidermis>")
				file.write("\t</medium>")
			if (matType == 2) and (matMediumType == 4):
				file.write("\t<medium>")
				file.write("\t\t<name>%s</name>"%(matname))
				file.write("\t\t<precedence>%g</precedence>"%(readValue(d, 'precedence', 1.0)))
				file.write("\t\t\t<atmosphere>")
				file.write("\t\t\t\t<turbidity>%s</turbidity>"%(readValue(d,'turbidity',2.2)))
				file.write("\t\t\t</atmosphere>")
				file.write("\t</medium>")
			#check if blended material
			if (matType == 3):
				matAName = readValue(d, 'blend.a_name', '')
				matBName = readValue(d, 'blend.b_name', '')
				# check if linked materials have already been exported -
				# if not do it now and add it infront of this material.
				try:
					exportedmatlist.index(matAName);
				except:
					matA = Blender.Material.Get(matAName)
					exportedmatlist.append(matAName)
					exportMaterial(matA, file, exportedmatlist)
				try:
					exportedmatlist.index(matBName);
				except:
					matB = Blender.Material.Get(matBName)
					exportedmatlist.append(matBName)
					exportMaterial(matB, file, exportedmatlist)
			matname = mat.getName()
			file.write("\t<material>\n")
			file.write("\t\t<name>%s</name>\n"%(matname))
			if (matType == 0):
				transparent=readValue(d,'diffuse.transmitter',0)
				if (transparent == 0):
					file.write("\t\t<diffuse>\n")
					file.write("\t\t\t<albedo_spectrum>\n\t\t\t\t<rgb>\n\t\t\t\t\t<rgb>%g %g %g</rgb>\n\t\t\t\t\t<gamma>2.2</gamma>\n\t\t\t\t</rgb></albedo_spectrum>\n"%(readValue(d, 'rgb.r', 0.0), readValue(d, 'rgb.g', 0.0), readValue(d, 'rgb.b', 0.0)))
				else:
					file.write("\t\t<diffuse_transmitter>\n")
					file.write("\t\t\t<albedo_spectrum>\n\t\t\t\t<rgb>\n\t\t\t\t\t<rgb>%g %g %g</rgb>\n\t\t\t\t\t<gamma>2.2</gamma>\n\t\t\t\t</rgb>\n\t\t\t</albedo_spectrum>\n"%(readValue(d, 'rgb.r', 0.0), readValue(d, 'rgb.g', 0.0), readValue(d, 'rgb.b', 0.0)))
			if (matType == 5):
				file.write("\t\t<oren_nayar>\n")
				file.write("\t\t\t<albedo_spectrum>\n\t\t\t\t<rgb>\n\t\t\t\t\t<rgb>%g %g %g</rgb>\n\t\t\t\t\t<gamma>2.2</gamma>\n\t\t\t\t</rgb>\n\t\t\t</albedo_spectrum>\n"%(readValue(d, 'rgb.r', 0.0), readValue(d, 'rgb.g', 0.0), readValue(d, 'rgb.b', 0.0)))
				file.write("\t\t\t<sigma>%f</sigma>\n"%readValue(d,'sigma',0.2))
			if (matType == 1):
				file.write("\t\t<phong>\n")
				file.write("\t\t\t<exponent>%g</exponent>\n"%(readValue(d, 'exponent', 1000.0)))
				if (readValue(d, 'nk_data', '').strip() != ''):
					#OS specific separator
					file.write("\t\t\t<nk_data>nkdata%s%s</nk_data>\n"%(os.sep,readValue(d, 'nk_data', '')))
				else:
					if not readValue(d,'toggle_phong',0.0):
						file.write("\t\t\t<diffuse_albedo_spectrum>\n\t\t\t\t<rgb>\n\t\t\t\t\t<rgb>%g %g %g</rgb>\n\t\t\t\t\t<gamma>2.2</gamma>\n\t\t\t\t</rgb>\n\t\t\t</diffuse_albedo_spectrum>\n"%(readValue(d, 'rgb.r', 0.0), readValue(d, 'rgb.g', 0.0), readValue(d, 'rgb.b', 0.0)))
					else:
						#file.write("\t\t\t<diffuse_albedo_spectrum>\n\t\t\t\t<rgb>\n\t\t\t\t\t<rgb>0 0 0</rgb>\n\t\t\t\t\t<gamma>2.2</gamma>\n\t\t\t\t</rgb>\n\t\t\t</diffuse_albedo_spectrum>\n")
						file.write("\t\t\t<specular_reflectivity_spectrum>\n\t\t\t\t<rgb>\n\t\t\t\t\t<rgb>%g %g %g</rgb>\n\t\t\t\t\t<gamma>1</gamma>\n\t\t\t\t</rgb>\n\t\t\t</specular_reflectivity_spectrum>\n"%(readValue(d, 'rgb.r_s', 0.0), readValue(d, 'rgb.g_s', 0.0), readValue(d, 'rgb.b_s', 0.0)))
					file.write("\t\t\t<ior>%g</ior>\n"%(readValue(d, 'ior', 1.0)))
			if (matType == 2):
				speculartype = readValue(d, 'specular.type', 1)
				if (speculartype == 1):
					file.write("\t\t<specular>\n")
				if (speculartype == 2):
					file.write("\t\t<glossy_transparent>\n")
					file.write("\t\t\t<exponent>%g</exponent>\n"%(readValue(d, 'exponent', 1000.0)))
				else:
					if (readValue(d, 'transparent', 0) != 0):
						file.write("\t\t\t<transparent>true</transparent>\n")
					else:
						file.write("\t\t\t<transparent>false</transparent>\n")					
				file.write("\t\t\t<internal_medium_name>%s</internal_medium_name>\n"%(matname))
			if (matType == 3):
				matAName = readValue(d, 'blend.a_name', '')
				matBName = readValue(d, 'blend.b_name', '')
				# check if linked materials have already been exported -
				# if not do it now and add it infront of this material.
				#try:
				#	exportedmatlist.index(matAName);
				#except:
				#	matA = Blender.Material.Get(matAName)
					#exportedmatlist.append(matAName)
					#exportMaterials(matA, file, exportedmatlist)
				#try:
				#	exportedmatlist.index(matBName);
				#except:
				#	matB = Blender.Material.Get(matBName)
					#exportedmatlist.append(matBName)
					#exportMaterials(matB, file, exportedmatlist)
				file.write("\t\t<blend>\n")
				file.write("\t\t\t<a_name>%s</a_name>\n"%(matAName))
				file.write("\t\t\t<b_name>%s</b_name>\n"%(matBName))
				file.write("\t\t\t<blend_factor>%g</blend_factor>\n"%(readValue(d, 'blend.factor', 0.5)))			
			if ((matType == 0) or (matType == 1 and not readValue(d,'toggle_phong',0) ) or (matType == 5)) and (readValue(d, 'albedo.path', '').strip() != ''):
				file.write("\t\t\t<albedo_texture>\n")
				print("Rel filename %s"%(relpath(file.name, readValue(d, 'albedo.path', ''))))
				file.write("\t\t\t\t<path>%s</path>\n"%(relpath(file.name, readValue(d, 'albedo.path', ''))))
				file.write("\t\t\t\t<uv_set>%s</uv_set>\n"%(readValue(d, 'albedo.uv_set', '')))
				file.write("\t\t\t\t<exponent>%f</exponent>\n"%(readValue(d, 'albedo.exponent', 1.0)))
				file.write("\t\t\t\t<a>%g</a>\n"%(readValue(d, 'albedo.a', 0.0)))
				file.write("\t\t\t\t<b>%g</b>\n"%(readValue(d, 'albedo.b', 1.0)))
				file.write("\t\t\t\t<c>%g</c>\n"%(readValue(d, 'albedo.c', 0.0)))
				file.write("\t\t\t</albedo_texture>\n")
			if ((matType == 0 and not readValue(d,'diffuse.transmitter',0)) or (matType == 1) or (matType == 2)) and (readValue(d, 'bump.path', '').strip() != ''):
				file.write("\t\t\t<bump_map>\n")
				file.write("\t\t\t\t<path>%s</path>\n"%(relpath(file.name, readValue(d, 'bump.path', ''))))
				file.write("\t\t\t\t<uv_set>%s</uv_set>\n"%(readValue(d, 'bump.uv_set', '')))
				file.write("\t\t\t\t<exponent>%f</exponent>\n"%(readValue(d, 'bump.exponent', 1.0)))
				file.write("\t\t\t\t<a>%g</a>\n"%(readValue(d, 'bump.a', 0.0)))
				file.write("\t\t\t\t<b>%g</b>\n"%(readValue(d, 'bump.b', 1.0)))
				file.write("\t\t\t\t<c>%g</c>\n"%(readValue(d, 'bump.c', 0.0)))
				file.write("\t\t\t</bump_map>\n")
			if (matType == 1) and (readValue(d, 'exponent.path', '').strip() != ''):
				file.write("\t\t\t<exponent_map>\n")
				file.write("\t\t\t\t<path>%s</path>\n"%(relpath(file.name, readValue(d, 'exponent.path', ''))))
				file.write("\t\t\t\t<uv_set>%s</uv_set>\n"%(readValue(d, 'exponent.uv_set', '')))
				file.write("\t\t\t\t<exponent>%f</exponent>\n"%(readValue(d, 'exponent.exponent', 1.0)))
				file.write("\t\t\t\t<a>%g</a>\n"%(readValue(d, 'exponent.a', 0.0)))
				file.write("\t\t\t\t<b>%g</b>\n"%(readValue(d, 'exponent.b', 1.0)))
				file.write("\t\t\t\t<c>%g</c>\n"%(readValue(d, 'exponent.c', 0.0)))
				file.write("\t\t\t</exponent_map>\n")
			if (matType == 3) and (readValue(d, 'blend.path', '').strip() != ''):
				file.write("\t\t\t<blend_map>\n")
				file.write("\t\t\t\t<path>%s</path>\n"%(relpath(file.name, readValue(d, 'blend.path', ''))))
				file.write("\t\t\t\t<uv_set>%s</uv_set>\n"%(readValue(d, 'blend.uv_set', '')))
				file.write("\t\t\t\t<exponent>%f</exponent>\n"%(readValue(d, 'blend.exponent', 1.0)))
				file.write("\t\t\t\t<a>%g</a>\n"%(readValue(d, 'blend.a', 0.0)))
				file.write("\t\t\t\t<b>%g</b>\n"%(readValue(d, 'blend.b', 1.0)))
				file.write("\t\t\t\t<c>%g</c>\n"%(readValue(d, 'blend.c', 0.0)))
				file.write("\t\t\t</blend_map>\n")	
			if (matType == 0):
				transparent=readValue(d,'diffuse.transmitter',0) #SmartDen
				if (transparent == 0): #SmartDen
					file.write("\t\t</diffuse>\n") #SmartDen
				else:
					file.write("\t\t</diffuse_transmitter>\n") #SmartDen
			if (matType == 5):
				file.write("\t\t</oren_nayar>\n")
			if (matType == 1):
				file.write("\t\t</phong>\n")
			if (matType == 2):
				speculartype = readValue(d, 'specular.type', 1)
				if (speculartype == 1):
					file.write("\t\t</specular>\n")
				if (speculartype == 2):
					file.write("\t\t</glossy_transparent>\n")
			if (matType == 3):
				file.write("\t\t</blend>\n")
			file.write("\t</material>\n")
			file.write("\n")
		if (matType == mInclude):
			includefile = readValue(d, 'include', '').strip()
			if includefile != '':
				includefile = relpath(file.name, includefile)
				print("Include file: %s"%includefile)
				print "including external file: \""+includefile+"\""
				try:
					includelist.index(includefile)
				except:
					includelist.append(includefile)
					file.write("\t<include>\n")
					file.write("\t\t<pathname>%s</pathname>\n"%includefile)
					file.write("\t</include>\n")
		if (matType == mLight) or (matType == mExitPortal):
			file.write("\t<material> <!-- light or exit portal dummy material -->\n")
			file.write("\t\t<name>%s</name>\n"%(matname))
			file.write("\t\t<null_material/>\n")
			file.write("\t</material>\n\n")



### ================================================================
### exportLight(material, pathname gain) -> string
###		returns indigo XML code for light material
### ----------------------------------------------------------------
def exportLight(mat, extgain, file):
	def readValue(dict, name, default):
		try:
			r = dict[name]
		except KeyError:
			r = default
		return r
	#End readValue
	if mat != None:
		try:
			d = mat.properties['indigo']
		except KeyError:
			d = {}
			print 'WARNING: material "%s" has no Indigo material definitions'%(mat.getName()) 
		m = []
		matType = readValue(d, 'type', 0)
		if matType == -1:
			lighttype = readValue(d, 'light.type', 1)
			gain = readValue(d, 'light.gain', 1.0) * extgain
			file.write("\t\t<spectrum>\n")
			if (lighttype == 1):
				file.write("\t\t\t<rgb>\n")
				file.write("\t\t\t\t<rgb>%g %g %g</rgb>\n"%(readValue(d, 'rgb.r', 0.0)*gain, readValue(d, 'rgb.g', 0.0)*gain, readValue(d, 'rgb.b', 0.0)*gain))
				file.write("\t\t\t\t<gamma>2.2</gamma>\n");
				file.write("\t\t\t</rgb>\n")
			if (lighttype == 2):
				file.write("\t\t\t<peak>\n")
				file.write("\t\t\t\t<base_value>%g</base_value>\n"%(readValue(d, 'light.base', 0.0)))
				file.write("\t\t\t\t<peak_min>%g</peak_min>\n"%(readValue(d, 'light.peak_min', 0.0)))
				file.write("\t\t\t\t<peak_width>%g</peak_width>\n"%(readValue(d, 'light.peak_width', 0.0)))
				file.write("\t\t\t\t<peak_value>%g</peak_value>\n"%(gain))
				file.write("\t\t\t</peak>\n")
			if (lighttype == 3):
				file.write("\t\t\t<blackbody>\n")
				temp = readValue(d, 'light.temperature', 1.0)
				file.write("\t\t\t\t<temperature>%g</temperature>\n"%(temp))
				# blackbody correction removed as indigo v0.7t3 has "efficacy scale" feature now
				# blackbody gain correction depending on temperature (see Stephan Boltzmann Law)
				file.write("\t\t\t\t<gain>%g</gain>\n"%(gain)) # *100/(5.6705e-8*pow(temp, 4))))
				file.write("\t\t\t</blackbody>\n")   
			file.write("\t\t</spectrum>\n")
			if (readValue(d, 'light.efficacy_scale', 0) != 0):
				file.write("\t\t<efficacy_scale>\n") 
				file.write("\t\t\t<power_drawn>%g</power_drawn>\n"%(readValue(d, 'light.power_drawn', 0.0))) 
				file.write("\t\t\t<overall_luminous_efficacy>%g</overall_luminous_efficacy>\n"%(readValue(d, 'light.overall_luminous_efficacy', 0.0)))
				file.write("\t\t</efficacy_scale>\n")
			elif (readValue(d,'light.ies',0) != 0):
				file.write("\t\t<ies_profile>\n")
				file.write("\t\t\t<path>%s</path>\n"%(relpath(file.name, readValue(d,'light.ies_path', ''))))
				file.write("\t\t</ies_profile>\n")
			if (readValue(d, 'albedo.path', '').strip() != ''):
				file.write("\t\t<texture>\n")
				file.write("\t\t\t<path>%s</path>\n"%(relpath(file.name, readValue(d, 'albedo.path', ''))))
				file.write("\t\t\t<uv_set>uv</uv_set>\n")
				file.write("\t\t\t<exponent>%f</exponent>\n"%(readValue(d, 'albedo.exponent', 1.0)))
				file.write("\t\t</texture>\n")



### ================================================================
### matIsLight(material) -> boolean
###		returns true if material is a light
### ----------------------------------------------------------------
def matIsLight(mat):
	def readValue(dict, name, default):
		try:
			r = dict[name]
		except KeyError:
			r = default
		return r
	if mat != None:
		try:
			d = mat.properties['indigo']
		except KeyError:
			d = {}
			print 'WARNING: material "%s" has no Indigo material definitions'%(mat.getName()) 
		return readValue(d, 'type', 0) == -1

### ================================================================
### matIsExitPortal(material) -> boolean
###		returns true if material is an exit portal
### ----------------------------------------------------------------
def matIsExitPortal(mat):
	def readValue(dict, name, default):
		try:
			r = dict[name]
		except KeyError:
			r = default
		return r
	if mat != None:
		try:
			d = mat.properties['indigo']
		except KeyError:
			d = {}
			print 'WARNING: material "%s" has no Indigo material definitions'%(mat.getName()) 
		return readValue(d, 'type', 0) == 4

### ================================================================
### matXXXXPathCB(filename)
###		GUI file-selector callbacks
### ----------------------------------------------------------------
def matLightIESPathCB(filename):
	global matLightIESPath, activeMaterial
	matLightIESPath.val = filename
	readWriteMaterial(activeMaterial, 1)

def matAlbedoPathCB(filename):
	global matAlbedoPath, activeMaterial
	matAlbedoPath.val = filename
	readWriteMaterial(activeMaterial, 1)
	
def matBumpPathCB(filename):
	global matBumpPath, activeMaterial
	matBumpPath.val = filename
	readWriteMaterial(activeMaterial, 1)

def matExponentPathCB(filename):
	global matExponentPath, activeMaterial
	matExponentPath.val = filename
	readWriteMaterial(activeMaterial, 1)

def matBlendPathCB(filename):
	global matBlendPath, activeMaterial
	matBlendPath.val = filename
	readWriteMaterial(activeMaterial, 1)

def matIncludeCB(filename):
	global matInclude, activeMaterial
	matInclude.val = filename
	readWriteMaterial(activeMaterial, 1)

### ================================================================
### materialGUI(y)
###		draws the material GUI from "y" position-up
### ----------------------------------------------------------------
materialGUIwidth = 440
def materialGUI(y):
	global matDiffTrans
	global matsMenu, activeMaterial, matType, matTypeMenu
	global matLightType, matLightGain, matLightBaseValue, matLightPeakMin, matLightPeakWidth, matLightTemp
	global matLightEfficacyScale, matLightIES, matLightIESPath, matLightPower, matLightEfficacy
	global matTogglePhong,matRgb, matRgbR, matRgbG, matRgbB, matRgb, matRgb_s, matRgbR_s, matRgbG_s, matRgbB_s, matRgbGain
	global matIOR, matExponent, matNKData
	global matSpecularType, matSigma, matTransparent, matMediumType, matPrecedence
	global matCauchyBCoeff, matAbsorptionType, matAbsR, matAbsG, matAbsB, matAbsGain, matAbs #BF
	global matSss, matSssType, matSssR, matSssG, matSssB, matSssRgb, matSssGain, matSssUniform, matSssUniformExp
	global matAlbedoUVset, matAlbedoPath, matAlbedoExponent, matAlbedoA, matAlbedoB, matAlbedoC
	global matBumpUVset, matBumpPath, matBumpExponent, matBumpA, matBumpB, matBumpC
	global matExponentUVset, matExponentPath, matExponentExponent, matExponentA, matExponentB, matExponentC
	global matBlendMatA, matBlendMatB
	global matBlendFactor, matBlendUVset, matBlendPath, matBlendExponent, matBlendA, matBlendB, matBlendC
	global matInclude, PreviewScene, matPreviewFile, PreviewName
	global matCommonMetals, matSssPhaseType, matSssPhaseSpec , matSssPhaseSpecUni, matSssPhaseSpecR, matSssPhaseSpecG, matSssPhaseSpecB
	global matHemoglobin, matTurbidity, matMelanin, matEumelanin

	drawButtons()

	def xd(pos, div):
		return (materialGUIwidth-10)/div*pos+10
	def wd(div):
		return (materialGUIwidth-10)/div-10
	y += 340
	mats = Blender.Material.Get()
	matsmenustr = "Materials:%t|"
	for m in mats:
		matsmenustr += m.getName()+"|"
	matsmenustr = matsmenustr[:-1]
	BGL.glColor3f(1.0,1.0,1.0); BGL.glRasterPos2i(xd(0,1),y+75); Draw.Text("Blender mat:")
	matsMenu = Draw.Menu(matsmenustr, evtMatsMenu, xd(0,1)+70, y+70, wd(2)-10, 18, matsMenu.val, "select Blender material")
	#BGL.glColor3f(1.0,1.0,1.0); BGL.glRasterPos2i(xd(0,1)+40,y+45); Draw.Text("Preset:")  #BF
	#matsPreset = Draw.Menu("Presets:%t|not yet!%x0|", evtNoEvt, xd(0,1)+90, y+40, 95, 18, matsPreset.val, "select a preset material")  #BF
	#Draw.Button("Save as", evtMatSaveas, xd(0,1)+185, y+40, 60, 18, "the present editor material")  #BF
	Draw.Button("Conv", evtMatConvert, xd(0,1)+wd(2)+60, y+70, 40, 18, "try converting blender material to indigo material")
	if activeMaterial != None:
		Draw.Button("Copy", evtMatCopy, xd(0,1)+wd(1)-155, y+50, 40, 18, "the present editor material")
		Draw.Button("Paste", evtMatPaste, xd(0,1)+wd(1)-115, y+50, 40, 18, "the  previously copied editor material")
		Draw.Button("Preview", evtMatPreview, xd(0,1)+wd(1)-50, y+50, 50, 18, "render a preview of the material in simple scene")

# Preview menu by BF
# All preview files use same material name "previewmaterial" - DaveC
		PreviewScene = Draw.Menu("Preview scene:%t|sphere%x0|cube%x1|monkey%x2|matDB%x4|custom - DIY!%x3", evtNoEvt, xd(0,1)+wd(1)-75, y+70, 75, 18, PreviewScene.val, "select a preview scene")  #BF
		if PreviewScene.val == 0:
			matPreviewFile = "IMPsphere.xml"
			PreviewName = "previewmaterial" #"IMPsphere" #commented out by DaveC
		if PreviewScene.val == 1:
			matPreviewFile = "IMPcube.xml"
			PreviewName = "previewmaterial" #"IMPcube"   #commented out by DaveC
		if PreviewScene.val == 2:
			matPreviewFile = "IMPmonkey.xml"
			PreviewName = "previewmaterial" #"IMPmonkey" #commented out by DaveC
		if PreviewScene.val == 3:
			matPreviewFile = "IMPcustom.xml"
			PreviewName = "previewmaterial" #"IMPcustom" #commented out by DaveC
		if PreviewScene.val == 4:
			matPreviewFile = "IMPmatdb.xml"
			PreviewName = "previewmaterial"

# old material buttons code removed BF
		BGL.glColor3f(1.0,1.0,1.0); BGL.glRasterPos2i(xd(0,1),y+55); Draw.Text("Indigo mat:")
		matTypeMenu = Draw.Menu("Material type:%t|EMITTER = lights+meshes%x-1|DIFFUSE = matt%x0|OREN-NAYAR = rough%x5|PHONG = glossy+metallic%x1|SPECULAR = transparent+sss%x2|BLENDED = 2 materials%x3|EXIT PORTAL%x4|EXTERNAL%x9", evtMatType, xd(0,1)+70, y+50, wd(2)-10, 18, matType, "select the desired material type")
		y -= 60
		if (matType == -1):
			BGL.glColor3f(1.0,1.0,1.0); BGL.glRasterPos2i(xd(0,1),y+85); Draw.Text("Emitter type:")
			matLightType = Draw.Menu("Light type:%t|RGB|peak|blackbody", evtNoEvt,xd(1,4)-35, y+80, wd(3)-20, 18, matLightType.val, "select light type")
			matLightGain = Draw.Number("Gain ", evtNoEvt, xd(1,4)-20, y+60, wd(4), 18, matLightGain.val, 1e-5, 1e5, "light-gain / peak-value")
			if (matLightType.val == 2): # peak
				matLightPeakMin = Draw.Number("p-min ", evtNoEvt, xd(1,2)-10, y+80, wd(4), 18, matLightPeakMin.val, 0.0, 1e15, "peak-min (nm)")
				matLightPeakWidth = Draw.Number("p-width ", evtNoEvt, xd(1,2)-10, y+60, wd(4), 18, matLightPeakWidth.val, 0.0, 1e15, "peak-width (nm)")
				matLightBaseValue = Draw.Number("Base ", evtNoEvt, xd(1,2)+wd(4)+10, y+80, wd(4), 18, matLightBaseValue.val, 0.0, 1e15, "base-value")
			if (matLightType.val == 3): # blackbody
				matLightTemp = Draw.Number("Temperature ", evtNoEvt, xd(1,2), y+60, wd(2), 18, matLightTemp.val, 0.0, 10000.0, "blackbody temperature K, sunlight 6500")
			matLightEfficacyScale = Draw.Toggle("Efficacy scale ", evtMatLightEfficacy, xd(1,2), y+20, wd(2), 18, matLightEfficacyScale.val, "enable efficacy scale")
			if matLightEfficacyScale.val != 0:
				matLightPower = Draw.Number("Power[W] ", evtNoEvt, xd(1,2), y, wd(2), 18, matLightPower.val, 0.0, 100000.0, "power drawn, range 0-100000")
				matLightEfficacy = Draw.Number("Efficacy[lm/W] ", evtNoEvt, xd(1,2), y-20, wd(2), 18, matLightEfficacy.val, 0.0, 1000.0, "a luminous efficiency, candle 0.3 flurotube 60 LPsodium 180")
			matLightIES = Draw.Toggle("Photometric",evtMatLightIES, xd(1,2), y-40, wd(2), 18, matLightIES.val, "Photometric emitter")
			if matLightIES.val == 1:
				BGL.glColor3f(1.0,1.0,1.0); BGL.glRasterPos2i(xd(0,1),y-55); Draw.Text("IES-File:")
				matLightIESPath = Draw.String("filename: ", evtNoEvt, xd(0,1)+70, y-60, wd(1)-90, 18, matLightIESPath.val, 250, "albedo texture filename (referencing Blender UV co-ords)")
				Draw.Button("...", evtMatLightIESPath, xd(0,1)+wd(1)-20, y-60, 20, 18, "open file selector")

			if (matLightType.val == 1): # peak
				BGL.glColor3f(1.0,1.0,1.0); BGL.glRasterPos2i(xd(0,1),y+45); Draw.Text("Emitter colour:")
			y -= 60
		if (matType == 0) or (matType == 5) or ((matType == -1) and (matLightType.val == 1)):
			matRgb = Draw.ColorPicker(evtMatRgbPicker, xd(0,2), y+40, 30, 58, (matRgbR.val, matRgbG.val, matRgbB.val), "colour and picker (LMB)")
			matRgbR = Draw.Slider("R ", evtNoEvt, xd(0,2)+30, y+80, wd(2)-30, 18, matRgbR.val, 0.0, 1.0, 1, "red component of colour")
			matRgbG = Draw.Slider("G ", evtNoEvt, xd(0,2)+30, y+60, wd(2)-30, 18, matRgbG.val, 0.0, 1.0, 1, "green component of colour")
			matRgbB = Draw.Slider("B ", evtNoEvt, xd(0,2)+30, y+40, wd(2)-30, 18, matRgbB.val, 0.0, 1.0, 1, "blue component of colour")
			if (matType == 0):
				matDiffTrans = Draw.Toggle("Transparent", evtNoEvt, xd(2,4), y+80, wd(4)+5, 18, matDiffTrans.val, "enable transparency")

		if (matType == 1):
			matTogglePhong=Draw.Toggle("Specular Reflectivity",evtNoEvt, xd(1,2),y+80,wd(2),18,matTogglePhong.val,"enable specular reflectivity color")
			if (matTogglePhong.val):
				matRgb_s = Draw.ColorPicker(evtMatRgbPicker_s, xd(0,2), y+40, 30, 58, (matRgbR_s.val, matRgbG_s.val, matRgbB_s.val), "colour and picker (LMB)")
				matRgbR_s = Draw.Slider("R ", evtNoEvt, xd(0,2)+30, y+80, wd(2)-30, 18, matRgbR_s.val, 0.0, 1.0, 1, "red component of colour")
				matRgbG_s = Draw.Slider("G ", evtNoEvt, xd(0,2)+30, y+60, wd(2)-30, 18, matRgbG_s.val, 0.0, 1.0, 1, "green component of colour")
				matRgbB_s = Draw.Slider("B ", evtNoEvt, xd(0,2)+30, y+40, wd(2)-30, 18, matRgbB_s.val, 0.0, 1.0, 1, "blue component of colour")
			else:
				matRgb = Draw.ColorPicker(evtMatRgbPicker, xd(0,2), y+40, 30, 58, (matRgbR.val, matRgbG.val, matRgbB.val), "colour and picker (LMB)")
				matRgbR = Draw.Slider("R ", evtNoEvt, xd(0,2)+30, y+80, wd(2)-30, 18, matRgbR.val, 0.0, 1.0, 1, "red component of colour")	
				matRgbG = Draw.Slider("G ", evtNoEvt, xd(0,2)+30, y+60, wd(2)-30, 18, matRgbG.val, 0.0, 1.0, 1, "green component of colour")
				matRgbB = Draw.Slider("B ", evtNoEvt, xd(0,2)+30, y+40, wd(2)-30, 18, matRgbB.val, 0.0, 1.0, 1, "blue component of colour")
				matNKData = Draw.String("NK data: ", evtNoEvt, xd(0,2), y-20, wd(2), 18, matNKData.val, 20, "lab measured mat. parameters called as e.g. Au.nk = gold")
			matExponent = Draw.Number("Exponent ", evtNoEvt, xd(0,2), y+20, wd(2), 18, matExponent.val, 1.0, 1000000.0, "surface shininess - rubber 50 , mirror 1 million")
			matIOR = Draw.Number("IOR ", evtNoEvt, xd(0,2), y, wd(2), 18, matIOR.val, 1e-4, 100.0, "index of refraction,water 1.33 glass 1.5")

		if (matType == 5):
			matSigma = Draw.Number("Sigma: ",evtNoEvt, xd(2,4), y+80,wd(4)+5,18,matSigma.val,0.0,1.0,"material roughness 0.0. - 1.0")

# common metals menu by BF
			#matCommonMetals = Draw.Menu("Common Metals:%t| not yet!%x1|aluminium%x2|brass%x3|bronze%x4|cadmium%x5|chrome%x6|copper%x7|gold%x8|iron%x9lead%x10|magnesiun%x11|molybdenum%x12|nickel%x13|platinum%x14|silver%x15|stainless%x16|tin%x17|titanium%x18|tungsten%x19|vanadium%x20", evtNoEvt, xd(1,2)+100, y-10, wd(2)-100, 18, matCommonMetals.val, "select a preset material")  #BF

		if (matType == 2):
			BGL.glColor3f(1.0,1.0,1.0); BGL.glRasterPos2i(xd(0,1),y+85); Draw.Text("Spec type:")
			matSpecularType = Draw.Menu("Specular type:%t|specular|glossy transparent", evtNoEvt, xd(1,4)-35, y+80, wd(3), 18, matSpecularType.val, "select specular type")
			if matSpecularType.val == 1: # specular
				matTransparent = Draw.Toggle("Transparent", evtNoEvt, xd(2,4), y+80, wd(4)+5, 18, matTransparent.val, "enable transparency")
			if matSpecularType.val == 2: # glossy
				matExponent = Draw.Number("Exponent ", evtNoEvt, xd(2,4), y+80, wd(4), 18, matExponent.val, 1.0, 1000000.0, "surface shininess - rubber 50 , mirror 1 million")				
# skin by BF
			BGL.glColor3f(1.0,1.0,1.0); BGL.glRasterPos2i(xd(0,1)+25,y+65); Draw.Text("Medium type:")
			matMediumType = Draw.Menu("Medium type:%t|basic%x1|dermis%x2|epidermis%x3|atmosphere%x4", evtNoEvt, xd(1,4), y+60, wd(4), 18, matMediumType.val, "select medium type")
			if matMediumType.val == 2: #dermis
				matHemoglobin = Draw.Number("Hemoglobin ", evtNoEvt, xd(1,2), y+40, wd(2), 18, matHemoglobin.val, 0.0, 1.0, "amount of hemoglobin, typ range .001-.01")	
			if matMediumType.val == 3: #epidermis
				matMelanin = Draw.Number("Melanin ", evtNoEvt, xd(2,4), y+40, wd(4)+5, 18, matMelanin.val, 0.0, 1.0, "amount of melanin, typ range 0-.05")
				matEumelanin = Draw.Number("Eumelanin ", evtNoEvt, xd(3,4)-5, y+40, wd(4)+5, 18, matEumelanin.val, 0.0, 1.0, "amount of eumelanin relative to pheomelanin")
			BGL.glColor3f(1.0,1.0,1.0); BGL.glRasterPos2i(xd(0,1)+25,y+45); Draw.Text("Precedence:")
			matPrecedence = Draw.Number("no. ", evtNoEvt, xd(1,4), y+40, wd(4), 18, matPrecedence.val, 1, 100, "set precendence of mediums when sharing same space")
			if matMediumType.val == 4:
				matTurbidity = Draw.Number("Turbidity", evtNoEvt, xd(1,2), y+40, wd(2), 18, matTurbidity.val, 2, 10, "Atmosphere turbidity. Values 2-10")	
			if matMediumType.val == 1: # basic #BF
				matCauchyBCoeff = Draw.Number("Cauchy B coeff ", evtNoEvt, xd(1,2), y+60, wd(2), 18, matCauchyBCoeff.val, 0.0, 1.0, "dispersive refr, range .003-.01, 0 disables ")
				matIOR = Draw.Number("IOR: ", evtNoEvt, xd(3,4)-5, y+80, wd(4)+5, 18, matIOR.val, 1e-4, 100.0, "index of refraction,water 1.33 glass 1.5")
				BGL.glColor3f(1.0,1.0,1.0); BGL.glRasterPos2i(xd(0,2),y+15); Draw.Text("Absorp type:")
				matAbsorptionType = Draw.Menu("Absorption type:%t|RGB%x1", evtNoEvt, xd(1,4)-35, y+10, wd(3), 18, matAbsorptionType.val, "select absorption type")
				if matAbsorptionType.val == 1:
					y -= 80
					matAbs = Draw.ColorPicker(evtMatAbsPicker, xd(1,2), y+50, 30, 58, (1.0-matAbsR.val, 1.0-matAbsG.val, 1.0-matAbsB.val), "absorption color and picker (LMB)")
					matAbsGain = Draw.Number("Gain ", evtNoEvt, xd(1,4), y+70, wd(4), 18, matAbsGain.val, 1e-3, 1e3, "RGB gain")
					matAbsR = Draw.Slider("R ", evtNoEvt, xd(1,2)+30, y+90, wd(2)-30, 18, matAbsR.val, 0.0, 1.0, 1, "red component of absorption colour")
					matAbsG = Draw.Slider("G ", evtNoEvt, xd(1,2)+30, y+70, wd(2)-30, 18, matAbsG.val, 0.0, 1.0, 1, "green component of absorption colour")
					matAbsB = Draw.Slider("B ", evtNoEvt, xd(1,2)+30, y+50, wd(2)-30, 18, matAbsB.val, 0.0, 1.0, 1, "blue component of absorption colour")
				y -= 20
				matSss = Draw.Toggle("Subsurface scattering", evtNoEvt, xd(0,2), y+50, wd(2), 18, matSss.val, "enable subsurface scattering")
				if matSss.val != 0:
					BGL.glColor3f(1.0,1.0,1.0); BGL.glRasterPos2i(xd(0,2),y+25); Draw.Text("Scatter type:")	
					matSssType = Draw.Menu("Scattering type:%t|RGB%x1|Uniform%x2", evtNoEvt, xd(1,4)-35, y+20, wd(3), 18, matSssType.val, "select scattering type")
					if matSssType.val == 1:
						y -= 60
						BGL.glColor3f(1.0,1.0,1.0); BGL.glRasterPos2i(xd(1,2),y+105); Draw.Text("SSS colour:")	
						matSssRgb = Draw.ColorPicker(evtMatSssRgbPicker, xd(1,2), y+40, 30, 58, (matSssR.val, matSssG.val, matSssB.val), "SSS color and picker (LMB)")
						matSssGain = Draw.Number("Gain ", evtNoEvt, xd(1,4), y+60, wd(4), 18, matSssGain.val, 1e-3, 1e3, "RGB gain")
						matSssR = Draw.Slider("R ", evtNoEvt, xd(1,2)+30, y+80, wd(2)-30, 18, matSssR.val, 0.0, 1.0, 1, "red component of SSS color")
						matSssG = Draw.Slider("G ", evtNoEvt, xd(1,2)+30, y+60, wd(2)-30, 18, matSssG.val, 0.0, 1.0, 1, "green component of SSS color")
						matSssB = Draw.Slider("B ", evtNoEvt, xd(1,2)+30, y+40, wd(2)-30, 18, matSssB.val, 0.0, 1.0, 1, "blue component of SSS color")
					if matSssType.val == 2:
						y -= 60
						matSssUniform = Draw.Number("Value",evtNoEvt, xd(1,2), y+80, wd(4), 18, matSssUniform.val,0,10,"scattering value mantissa")
						matSssUniformExp = Draw.Number("x10^ ",evtNoEvt, xd(3,4), y+80, wd(4), 18, matSssUniformExp.val,-5,5,"scattering value exponent 10^+-5")
					BGL.glColor3f(1.0,1.0,1.0); BGL.glRasterPos2i(xd(1,4)-30,y+20); Draw.Text("Phase function:")
					matSssPhaseType = Draw.Menu("Phase type:%t|Uniform%x1|Henyey Greenstein%x2", evtNoEvt, xd(2,4)-35, y+15, wd(3), 18, matSssPhaseType.val, "select phase type")
					if matSssPhaseType.val == 2:
						#matSssPhaseFunct = Draw.Number("'g' ",evtNoEvt, xd(3,4), y+15, wd(4), 18, matSssPhaseFunct.val,-1.0,1.0,"Henyey-Greenstein phase function ")  #BF
						matSssPhaseSpec = Draw.Menu("Phase Type Spectrum:%t|Uniform%x1|RGB%x2",evtNoEvt,xd(3,4),y+15,wd(4),18,matSssPhaseSpec.val,"Select phase spectrum")
						if matSssPhaseSpec.val == 1:
							y-=15
							matSssPhaseSpecUni=Draw.Number("Value",evtNoEvt,xd(3,4),y+10,wd(4),18,matSssPhaseSpecUni.val,-1,1,"Phase Function Uniform Value")
						if matSssPhaseSpec.val == 2:
							y-=15
							matSssPhaseSpecR=Draw.Number("R",evtNoEvt,xd(1,4),y+10,wd(4),18,matSssPhaseSpecR.val,0,1,"Phase Function Red Value")
							matSssPhaseSpecG=Draw.Number("G",evtNoEvt,xd(2,4),y+10,wd(4),18,matSssPhaseSpecG.val,0,1,"Phase Function Green Value")
							matSssPhaseSpecB=Draw.Number("B",evtNoEvt,xd(3,4),y+10,wd(4),18,matSssPhaseSpecB.val,0,1,"Phase Function Blue Value")
						y+=30
					else:
						y+=15


		if (matType == 3):
			BGL.glRasterPos2i(xd(0,2), y+35); Draw.Text("Material A: "+matBlendMatA.val)
			Draw.Button("...", evtMatBlendMatA, xd(0,2)+wd(2)-40, y+30, 40, 18, "select a material from list")
			BGL.glRasterPos2i(xd(1,2), y+35); Draw.Text("Material B: "+matBlendMatB.val)
			Draw.Button("...", evtMatBlendMatB, xd(1,2)+wd(2)-40, y+30, 40, 18, "select a material from list")
			matBlendFactor = Draw.Slider("Blend-factor ", evtNoEvt, xd(0,1), y+10, wd(1), 18, matBlendFactor.val, 0.0, 1.0, 0, "blend ratio of the materials")
			y -= 60
			BGL.glColor3f(1.0,1.0,1.0); BGL.glRasterPos2i(xd(0,1),y+45); Draw.Text("Blend map:")
			matBlendPath = Draw.String("filename: ", evtNoEvt, xd(0,1)+70, y+40, wd(1)-90, 18, matBlendPath.val, 250, "blend texture filename")
			kk = Draw.Button("...", evtMatBlendPath, xd(0,1)+wd(1)-20, y+40, 20, 18, "open file selector")			
			matBlendExponent = Draw.Number("gamma ", evtNoEvt, xd(0,2), y+20, wd(4), 18, matBlendExponent.val, 0.0, 10.0, "texture gammma")
			matBlendUVset = Draw.String("uv-set ", evtNoEvt, 107, y+20, 95, 18, matBlendUVset.val, 20, "blend texture UV-set")
			matBlendA = Draw.Number("A ", evtNoEvt, xd(2,5)+20, y+20, wd(5), 18, matBlendA.val, 0.0, 10.0, "")
			matBlendB = Draw.Number("B ", evtNoEvt, xd(3,5)+10, y+20, wd(5), 18, matBlendB.val, 0.0, 10.0, "")
			matBlendC = Draw.Number("C ", evtNoEvt, xd(4,5), y+20, wd(5), 18, matBlendC.val, 0.0, 10.0, "")
		if (matType == 0) or (matType == 1 and not matTogglePhong.val) or (matType == 5) or(matType == -1):
			y -= 80
			BGL.glColor3f(1.0,1.0,1.0); BGL.glRasterPos2i(xd(0,1),y+35); Draw.Text("Albedo:")
			matAlbedoPath = Draw.String("filename: ", evtNoEvt, xd(0,1)+70, y+30, wd(1)-90, 18, matAlbedoPath.val, 250, "albedo texture filename (referencing Blender UV co-ords)")
			Draw.Button("...", evtMatAlbedoPath, xd(0,1)+wd(1)-20, y+30, 20, 18, "open file selector")			
			matAlbedoExponent = Draw.Number("gamma ", evtNoEvt, xd(0,2), y+10, wd(4), 18, matAlbedoExponent.val, 0.0, 10.0, "texture gammma")

			matAlbedoUVset = Draw.String("UVset ", evtNoEvt, 107, y+10, 95, 18, matAlbedoUVset.val, 20, "albedo texture UV-set")
			if (matType >= 0):
				matAlbedoA = Draw.Number("A ", evtNoEvt, xd(2,5)+20, y+10, wd(5), 18, matAlbedoA.val, 0.0, 10.0, "")
				matAlbedoB = Draw.Number("B ", evtNoEvt, xd(3,5)+10, y+10, wd(5), 18, matAlbedoB.val, 0.0, 10.0, "")
				matAlbedoC = Draw.Number("C ", evtNoEvt, xd(4,5), y+10, wd(5), 18, matAlbedoC.val, 0.0, 10.0, "")
		if (matType == 0 and not matDiffTrans.val) or (matType == 1) or (matType == 2) or (matType == 5):
			y -= 80
			BGL.glColor3f(1.0,1.0,1.0); BGL.glRasterPos2i(xd(0,1),y+35); Draw.Text("Bump map:")
			matBumpPath = Draw.String("filename: ", evtNoEvt, xd(0,1)+70, y+30, wd(1)-90, 18, matBumpPath.val, 250, "bump texture filename (referencing Blender UV co-ords)")
			Draw.Button("...", evtMatBumpPath, xd(0,1)+wd(1)-20, y+30, 20, 18, "open file selector")			
			matBumpExponent = Draw.Number("gamma ", evtNoEvt, xd(0,2), y+10, wd(4), 18, matBumpExponent.val, 0.0, 10.0, "texture gammma")
			matBumpUVset = Draw.String("UVset ", evtNoEvt, 107, y+10, 95, 18, matBumpUVset.val, 20, "bump texture UV-set")
			matBumpA = Draw.Number("A ", evtNoEvt, xd(2,5)+20, y+10, wd(5), 18, matBumpA.val, 0.0, 1.0, "")
			matBumpB = Draw.Number("B ", evtNoEvt, xd(3,5)+10, y+10, wd(5), 18, matBumpB.val, -1.0, 1.0, "set bump height here typically 0.001")
			BGL.glColor3f(0.9,0.9,0.9); BGL.glRectf( xd(3,5)-5, y+18, xd(4,5)-25, y+19)
			matBumpC = Draw.Number("C ", evtNoEvt, xd(4,5), y+10, wd(5), 18, matBumpC.val, 0.0, 1.0, "")
		if (matType == 1):
			y -= 70
			#BGL.glColor3f(1.0,1.0,1.0); BGL.glRasterPos2i(xd(0,1),y+235); Draw.Text("Phong colour:")	
			BGL.glColor3f(1.0,1.0,1.0); BGL.glRasterPos2i(xd(0,1),y+55); Draw.Text("Exponent:")
			matExponentPath = Draw.String("filename: ", evtNoEvt, xd(0,1)+70, y+50, wd(1)-90, 18, matExponentPath.val, 250, "exponent texture filename (referencing Blender UV co-ords)")
#			BGL.glRasterPos2i(xd(0,1)+70, y+45); Draw.Text("filename: "+matExponentPath.val)
			Draw.Button("...", evtMatExponentPath, xd(0,1)+wd(1)-20, y+50, 20, 18, "open file selector")			
			matExponentExponent = Draw.Number("gamma ", evtNoEvt, xd(0,2), y+30, wd(4), 18, matExponentExponent.val, 0.0, 10.0, "texture gammma")
			matExponentUVset = Draw.String("UVset ", evtNoEvt, 107, y+30, 95, 18, matExponentUVset.val, 20, "exponent texture UV-set")
			matExponentA = Draw.Number("A ", evtNoEvt, xd(2,5)+20, y+30, wd(5), 18, matExponentA.val, 0.0, 10000.0, "")
			matExponentB = Draw.Number("B ", evtNoEvt, xd(3,5)+10, y+30, wd(5), 18, matExponentB.val, 0.0, 10000.0, "")
			matExponentC = Draw.Number("C ", evtNoEvt, xd(4,5), y+30, wd(5), 18, matExponentC.val, 0.0, 10000.0, "")
		if (matType == mInclude):
			BGL.glColor3f(1.0,1.0,1.0); BGL.glRasterPos2i(xd(0,1),y+35); Draw.Text("Include file:")
			matInclude = Draw.String("filename: ", evtNoEvt, xd(0,1)+70, y+30, wd(1)-90, 18, matInclude.val, 250, "include filename")
			Draw.Button("...", evtMatInclude, xd(0,1)+wd(1)-20, y+30, 20, 18, "open file selector")

### ================================================================
### materialEvent(evt, val)
###		material GUI event routine
### ----------------------------------------------------------------
def materialEvent(evt, val):
	global activeObject, activeObjectMat0, matsMenu, activeMaterial
	#obj = Blender.Scene.GetCurrent().getActiveObject()
	obj = Blender.Scene.GetCurrent().objects.active
	if(obj != None):
		try:
			try:
				mat0 = (obj.getData(0, 1).materials)[0]
			except IndexError:
				mat0 = None
		except AttributeError:
			mat0 = None
	else: mat0 = None
	if (activeObject != obj) or (activeObjectMat0 != mat0):
		activeObject = obj
		activeObjectMat0 = mat0
		if (mat0 != None):
			mats = Blender.Material.Get()
			matsMenu.val = 0
			matindex = matsMenu.val
			for m in mats:
				if m.getName() == mat0.getName():
					matindex = mats.index(m)+1
					activeMaterial = m
			matsMenu.val = matindex
			readWriteMaterial(activeMaterial, 0)
			drawGUI()
			Draw.Redraw(1)


### ================================================================
### materialButtonEvt(evt)
###		material GUI button-event routine
### ----------------------------------------------------------------
def materialButtonEvt(evt):
	global matsMenu, matType, activeMaterial, matBlendMatA, matBlendMatB, matSssRgb, matSssUniform, matClipboard
	#, matsPreset
	global matAbsR, matAbsG, matAbsB, matRgbR, matRgbG, matRgbB, matSssR, matSssG, matSssB
	if evt == evtMatsMenu:
		mats = Blender.Material.Get()
		activeMaterial = mats[matsMenu.val-1]
		readWriteMaterial(activeMaterial, 0)
	if evt == evtMatType:
		matType = matTypeMenu.val
	if evt == evtMatAbsPicker:
		matAbsR.val = 1.0-matAbs.val[0]
		matAbsG.val = 1.0-matAbs.val[1]
		matAbsB.val = 1.0-matAbs.val[2]
	if evt == evtMatRgbPicker:
		matRgbR.val = matRgb.val[0]
		matRgbG.val = matRgb.val[1]
		matRgbB.val = matRgb.val[2]
	if evt == evtMatRgbPicker_s:
		matRgbR_s.val = matRgb_s.val[0]
		matRgbG_s.val = matRgb_s.val[1]
		matRgbB_s.val = matRgb_s.val[2]
	if evt == evtMatSssRgbPicker:
		matSssR.val = matSssRgb.val[0]
		matSssG.val = matSssRgb.val[1]
		matSssB.val = matSssRgb.val[2]
	if (evt == evtMatBlendMatA) or (evt == evtMatBlendMatB):
		mats = Blender.Material.Get()
		matsmenustr = "materials:%t|"
		for m in mats:
			matsmenustr += m.getName()+"|"
		matsmenustr += "Default|null"
		i = Draw.PupMenu(matsmenustr)
		if i > 0:
			if (i <= len(mats)):
				matname = mats[i-1].getName()
			else:
				if (i == len(mats)+1):
					matname = "Default"
				else:
					matname = "null"
			if (evt == evtMatBlendMatA):
				matBlendMatA.val = matname
			if (evt == evtMatBlendMatB):
				matBlendMatB.val = matname
	if evt == evtMatLightIESPath:
		Blender.Window.FileSelector(matLightIESPathCB,'select IES file')
	if (evt == evtMatAlbedoPath):
		Blender.Window.FileSelector(matAlbedoPathCB, 'select texture')
	if (evt == evtMatBumpPath):
		Blender.Window.FileSelector(matBumpPathCB, 'select texture')
	if (evt == evtMatExponentPath):
		Blender.Window.FileSelector(matExponentPathCB, 'select texture')
	if (evt == evtMatBlendPath):
		Blender.Window.FileSelector(matBlendPathCB, 'select texture')
	if (evt == evtMatInclude):
		Blender.Window.FileSelector(matIncludeCB, 'select include-file')
	if (evt == evtMatPreview):
		if (matType >= 0) and (matType != mInclude):
			exportMatPreview(activeMaterial)
		else:
			Draw.PupMenu("can't preview emitter or external materials, sorry%t|OK%x1")
#	if (evt == evtMatPreviewFile):
#		if (matType >= 0) and (matType != mInclude):
#			matPreviewFile = Draw.PupStrInput("Material preview filename:", matPreviewFile, 100)
#BF		exportMatPreview(activeMaterial)
#		else:
#			Draw.PupMenu("Can't preview light- or external-materials, sorry%t|OK%x1")
	#if (evt == evtMatsPreset):  #BF
	#	matPreset = matPreset.val  #BF
	if (evt == evtMatCopy):
		if (activeMaterial != None):
			try:
				readWriteMaterial(activeMaterial, 1)
				matClipboard = activeMaterial.properties['indigo'].convert_to_pyobject()
			except KeyError:
				print 'WARNING: material "%s" has no Indigo material definitions'%(activeMaterial) 
	if (evt == evtMatPaste) and (matClipboard != None):
		if (activeMaterial != None):
			try:
				activeMaterial.properties['indigo'] = matClipboard
				readWriteMaterial(activeMaterial, 0)
			except KeyError:
				print 'WARNING: material "%s" has no Indigo material definitions'%(activeMaterial) 
	if (evt == evtMatConvert):
		if (activeMaterial != None):
			convertMaterial(activeMaterial);
			readWriteMaterial(activeMaterial, 0);
	readWriteMaterial(activeMaterial, 1)
#	Draw.Redraw()



# Draw.Register(gui, materialEvent, materialButtonEvt)

### ----------------------------------------------------------------
### end of INDIGO material editor module - (by zuegs)
########################################################################
########################################################################
########################################################################









######################################################
# Functions
######################################################


# New name based on old with a different extension
def newFName(ext):
	return Blender.Get('filename')[: -len(Blender.Get('filename').split('.', -1)[-1]) ] + ext

def IndigoBoolean(val):
	if val != 0:
		val = "true"
	else:
		val = "false"
	return val

def splitn(t):
	nt = t.split('\n')
	return nt[0]

def colGamma(value):
	global ColExponent
	return value**ColExponent.val


######################################################
# EXPORT
######################################################

def launchIndigo(filename):
	if linux:
		f = open(os.path.join(indigopath, "IndigoWrapper.sh"), 'w')
		f.write("#!/bin/sh\n") 
		f.write("cd \"" + os.path.join(indigopath, "") + "\"\n")
		if os.path.isfile(os.path.join(indigopath,"indigo.exe")):
			f.write("nice -n 20 wine indigo.exe \"" + relpath(indigopath,filename) + "\" -t " + str(Threads.val))
		elif os.path.isfile(os.path.join(indigopath,"indigo_console")):
			f.write("nice -n 20 xterm -e ./indigo_console \"" + relpath(indigopath, filename) + "\" -t " + str(Threads.val))
		else:
			f.write("echo Indigo can not be found in %s!"%indigopath)
		f.close()
		os.system("chmod +x \"" + os.path.join(indigopath, "IndigoWrapper.sh") + "\"")
		# call external shell script to start Indigo || linux
		cmd= "\"" + os.path.join(indigopath, "IndigoWrapper.sh") + "\" &"
	else:
		f = open(os.path.join(indigopath, 'IndigoWrapper.cmd'), 'w') 
		f.write("cd /d \"" + os.path.join(indigopath, "") + "\"\n") 
		print "\""+indigopath+"\" \""+filename+"\""
		if UseGUI.val:
			f.write("start /b /belownormal indigo.exe \"" + relpath(indigopath,filename) + "\n")
		else:
			f.write("start /belownormal cmd.exe /c indigo_console.exe \"" + relpath(indigopath,filename) + "\" -t " + str(Threads.val) +  "\n")
		f.close() 
		# call external shell script to start Indigo || win
		cmd = "\"" + os.path.join(indigopath, "IndigoWrapper.cmd") + "\""
	print cmd
	os.system(cmd)

def launchAnimIndigo(filename):
	f = open(os.path.join(indigopath, 'AnimFrames.cmd'), 'w')
	f.write("@echo off\n")
	f.write("cd /d \"" + os.path.join(indigopath, "") + "\"\n") 
	startF = Blender.Get('staframe')
	endF = Blender.Get('endframe')
	f.write('echo Start rendering Animation\n')
	f.write('echo Start Frame: '+str(startF)+'\n')
	f.write('echo End Frame: '+str(endF)+'\n')
	f.write('echo Total Frames: '+str(endF-startF+1)+'\n')
	if UseGUI.val:
		for i in range (startF, endF+1):
			f.write('echo Rendering Frame '+str(i)+'\n')
			f.write("start /min /belownormal /wait indigo.exe \""+relpath(indigopath, makename(filename, '-'+str(i)+'.igs'))+"\" -t "+str(Threads.val)+" -o \""+relpath(indigopath, makename(filename,'-'+str(i)+'.png'))+"\"\n")
	else:
		for i in range (startF, endF+1):
			f.write("start /min /belownormal /wait cmd.exe /c indigo_console.exe "+relpath(indigopath, makename(filename, '-'+str(i)+'.igs'))+" -o "+relpath(indigopath, makename(filename,'-'+str(i)+'.png'))+"\n")
	f.close()
	
	f=open(os.path.join(indigopath, "startAnim.cmd"), 'w')
	f.write("cd /d \"" + os.path.join(indigopath, "") + "\"\n") 
	f.write("start AnimFrames.cmd"+"\n")
	f.close
	# call external shell script to start Indigo || win
	cmd = "start cmd /c \"" + os.path.join(indigopath, "startAnim.cmd") + "\""
	print cmd
	os.system(cmd)




### END OF PSOR ##########################################################


def getBoundingBox(mesh):
	bb = [[0.0, 0.0], [0.0, 0.0], [0.0, 0.0]]
	if len(mesh.verts) > 0:
		v = mesh.verts[0]
		for i in range(3):
			bb[i][0] = v[i]
			bb[i][1] = v[i]
		for v in mesh.verts:
			for i in range(3):
				if bb[i][0] < v[i]:
					bb[i][0] = v[i]
				if bb[i][1] > v[i]:
					bb[i][1] = v[i]
	return bb

		

def exportMesh(meshblock, matlist, file):
	xml = []
	meshname = meshblock[0]
	mesh = meshblock[1]
	meshmatrix = meshblock[2]
	print 'exporting mesh "%s"'%meshname
	vdata = []   # list of [ii0, ii1, ii2, ...] lists indexed by Blender-Vertex-index
	vlist = []   # list of [coord, normal, uv] lists indexed by Indigo-Vertex-index
	flist = []

	def addVertex(bvindex, coord, normal, uv):
		index = -1
		if (bvindex < len(vdata)):
			for ivindex in vdata[bvindex]:
				v = vlist[ivindex]
				if (abs(v[0][0]-coord[0])<0.0001) and \
				(abs(v[0][1]-coord[1])<0.0001) and \
				(abs(v[0][2]-coord[2])<0.0001) and \
				(abs(v[1][0]-normal[0])<0.0001) and \
				(abs(v[1][1]-normal[1])<0.0001) and \
				(abs(v[1][2]-normal[2])<0.0001):
					if ((v[2]==[]) and (uv==[])) or \
					((abs(v[2][0]-uv[0])<0.0001) and \
					(abs(v[2][1]-uv[1])<0.0001)):
						index = ivindex
		if index < 0:
			index = len(vlist)
			vlist.append([coord, normal, uv])
			while bvindex >= len(vdata):
				vdata.append([])
			vdata[bvindex].append(index)
		return index

	def getVertices():
		def getUVStr(uv):
			if uv != []: return " uv0=\"%f %f\"" %(uv[0], uv[1])
			else: return ""
		for v in vlist:
			vertex="\t\t\t<vertex pos=\"%f %f %f\" normal=\"%f %f %f\"%s />\n" \
			%(v[0][0], v[0][1], v[0][2], v[1][0], v[1][1], v[1][2], getUVStr(v[2]))
			file.write(vertex)
		#file.write(map( \
		#	lambda v: "\t\t\t<vertex pos=\"%f %f %f\" normal=\"%f %f %f\"%s />\n" \
		#	%(v[0][0], v[0][1], v[0][2], v[1][0], v[1][1], v[1][2], getUVStr(v[2])), \
		#	vlist))

	def addFace(mindex, index0, index1, index2):
		while mindex >= len(flist):
			flist.append([])
		flist[mindex].append([index0, index1, index2])

	def getFaces():
		def getMaterialName(mindex):
			if mindex < len(mesh.materials):
				matname = mesh.materials[mindex].name
				try:
					matlist.index(matname)
				except ValueError:		  
					matlist.append(matname)
				return matname
			else:
				print "WARNING: faces without material"
				return "Default"
		for fli in range(len(flist)):
			tri_set="\t\t\t<triangle_set>\n\t\t\t\t<material_name>%s</material_name>\n"%(getMaterialName(fli))
			file.write(tri_set)
			for f in flist[fli]:
				tris="\t\t\t\t<tri>%d %d %d</tri>\n"%(f[0], f[1], f[2])
				file.write(tris)
			file.write("\t\t\t</triangle_set>\n")
	
	if len(mesh.faces) > 0:
		for face in mesh.faces:
			iis = [-1, -1, -1, -1]
			for vi in range(len(face.v)):
				vert = face.v[vi]
				if face.smooth:
					normal = vert.no.normalize()
				else:
					normal = face.no.normalize()
				#if len(face.uv) == len(face.v):
				if mesh.faceUV:
					uv = face.uv[vi]
				else:
					uv = []
				if meshmatrix:
					coord = Mathutils.Vector(vert.co) * meshmatrix
					normal2 = Mathutils.Vector(normal) * meshmatrix
					iis[vi] = addVertex(vert.index, coord, normal2, uv)
				else:
					iis[vi] = addVertex(vert.index, vert.co, normal, uv)
			#addFace(face.materialIndex, iis[0], iis[1], iis[2])
			addFace(face.mat, iis[0], iis[1], iis[2])
			if len(face.v)==4:
				#addFace(face.materialIndex, iis[2], iis[3], iis[0])
				addFace(face.mat, iis[2], iis[3], iis[0])
	else:
		print 'WARNING: mesh "%s" has no faces!'%meshname
	
	file.write("\t<mesh>\n")
	file.write("\t\t<name>%s</name>\n" %(meshname))
	if len(mesh.faces) > 0:
		file.write("\t\t<normal_smoothing>%s</normal_smoothing>\n" %(IndigoBoolean(face.smooth)))
		file.write("\t\t<embedded>\n")
		file.write("\t\t\t<expose_uv_set>\n")
		file.write("\t\t\t\t<index>0</index>\n")
		file.write("\t\t\t\t<name>uv</name>\n")
		file.write("\t\t\t</expose_uv_set>\n")
		getVertices()
		getFaces()
		file.write("\t\t</embedded>\n")
	else:
		file.write("\t\t<normal_smoothing>false</normal_smoothing>\n")
		file.write("\t\t<embedded>\n")
#		file.write("\t\t\t<triangle_set>\n")
#		file.write("\t\t\t\t<material_name>null</material_name>\n")
#		file.write("\t\t\t</triangle_set>\n")
		file.write("\t\t</embedded>\n")
	file.write("\t</mesh>\n\n")
	
#	return "".join(xml)

	

#def exportMeshs(meshlist, matlist , meshsfn = ''):
#	xml = []
#	if meshsfn != '':
#		file=open(meshsfn,'w')
#		file.write("<?xml version=\"1.0\" encoding=\"utf-8\"?>\n\n")
#		file.write('<scenedata>\n')
#		for mb in meshlist:
#			exportMesh(mb, matlist, file)
#		file.write('</scenedata>\n')
#		file.close()
		


#def exportObject(obj, matrix, meshlist, matlist, filename):
def exportObject(obj, meshlist, matlist, file):
	def sign(x):
		s = 0
		if x>1e-6: s = 1
		if x<-1e-6: s = -1
		return s
#	xml = []
	matrix=obj.getMatrix()
	objectname = obj.getName()
	objecttype = obj.getType()
	objectdup = obj.enableDupGroup or obj.enableDupVerts
	ws = WorldScale.val*(10**WorldScaleExp.val)
	if (objecttype == "Mesh") or (objecttype == "Curve") or (objecttype == "Text"):
	#if ((objecttype == "Mesh") and not(objectdup)):
		print('exporting object "%s"'%(objectname))
		meshname=obj.getData(name_only=1)
		if(objecttype == "Curve"):
			meshname +='_cu'

		mesh=getMeshFromObject(obj)
		if len(mesh.faces) <= 0:
			print "skip object"
			return
		if (objecttype == "Curve") or (objecttype == "Text"):
			for f in mesh.faces:
				f.smooth = 0
		if (objecttype == "Curve"):
		#	mesh.setMaterials(obj.getData().getMaterials())
			mesh.materials = obj.getData().getMaterials()
		meshlight = 0
		exitportal = 0
		mat_idx=0
		if len(mesh.materials) > 0:
			for mat_i in mesh.materials:
				#mat0 = mesh.materials[0]
				print("material %s found\n"%mat_i)
				if matIsLight(mat_i):
					if mat_idx > 0:
						print("Multiple material meshlight!")
						#Blender.Draw.PupMenu("Error: Multiple materila meshlight!%t|Ok")
						#raise("Error: Multiple material meshlight not allowed!\nMeshlight: %s"%mesh.name)
						#sys.exit()
					meshlight = 1
				elif matIsExitPortal(mat_i):
					exitportal = 1
				#else:
					#try:
					#	matlist.index(mat_i.name)
					#except ValueError:		  
					#	matlist.append(mat_i.name)
				mat_idx=mat_idx+1
		if len(mesh.faces) <= 0:
			print 'WARNING: ignoring object "%s", because linked mesh "%s" has no faces'%(objectname, meshname)
			return ""
		file.write("\t<!-- %s -->\n" %(objectname.replace("<","-").replace(">","-")))
		imeshname = meshname.replace("<","-").replace(">","-")
		pos = matrix.translationPart()
		scl = matrix.scalePart()
    # rot = matrix.toEuler().toMatrix()
    # the blender euler rotation is wrong in some cases and don't match indigo rotation so here a new one:
		rot = Blender.Mathutils.Matrix(matrix[0]/scl[0],matrix[1]/scl[1],matrix[2]/scl[2])
		# if scale is non-uniform we will export a mesh per object with the specific scale - for this create a matrix "mat" that
		# holds transformation to apply to mesh.
		# note: non-uniform scaled objects could not be instanced as they use their own spacial scaled mesh
		# if you want instancing you have to apply all scalings to the underlying blender-mesh before export (ctrl-A)

		# process indigo primitives
		if objectname.find(".sphere") != -1:
			bb = getBoundingBox(mesh)
			radius = ws * 0.5 * min(abs(scl[0]*(bb[0][1]-bb[0][0])), abs(scl[1]*(bb[1][1]-bb[1][0])), abs(scl[2]*(bb[2][1]-bb[2][0])))
			print 'Exporting Object %s as sphere'%objectname
			file.write("\t<sphere>\n")
			file.write("\t\t<center>%f %f %f</center>\n"%(pos[0]*ws, pos[1]*ws, pos[2]*ws))
			file.write("\t\t<radius>%f</radius>\n"%radius)
			file.write("\t\t<material_name>%s</material_name>\n"%mat0.name)
			file.write("\t</sphere>\n\n")
		elif objectname.find('.plane') != -1:
			dist =ws* pos[0]*rot[2][0]+pos[1]*rot[2][1]+pos[2]*rot[2][2]
			file.write("\t<plane>\n")
			file.write("\t\t<normal>%f %f %f</normal>\n"%(rot[2][0], rot[2][1], rot[2][2]))
			file.write("\t\t<dist>%f</dist>\n"%dist)
			file.write("\t\t<material_name>%s</material_name>\n"%mat0.name)
			file.write("\t</plane>\n\n")
		else: #not sphere and not plane
		#	print "Mesh"
			if (abs(scl[0]-scl[1])>1e-4) or (abs(scl[0]-scl[2])>1e-4):
				mat = Blender.Mathutils.Matrix()
				for v in range(3):
					for z in range(3):
						mat[z][v] = matrix[z][v]
				rot = Blender.Mathutils.Matrix()
				scalefact = 1.0
			else:
				scalefact = (scl[0] + scl[1] + scl[2]) / 3
			if meshlight:
				file.write("\t<meshlight>\n")
			elif exitportal:
				file.write("\t<exit_portal>\n")
			else:
				file.write("\t<model>\n")
			file.write("\t\t<pos>%f %f %f</pos>\n" %(pos[0]*ws, pos[1]*ws, pos[2]*ws))
			file.write("\t\t<scale>%g</scale>\n" %(scalefact*ws))
			file.write("\t\t<rotation>\n")
			file.write("\t\t\t<matrix>\n")
			file.write("\t\t\t\t")
			for v in range(3):
				for z in range(3):
					file.write("%f " % (rot[z][v]))
			file.write("\n")
			file.write("\t\t\t</matrix>\n")
			file.write("\t\t</rotation>\n")
			file.write("\t\t<mesh_name>%s</mesh_name>\n" %(imeshname))
			if meshlight:	
				gain = 1.0
				exportLight(mesh.materials[0], gain, file)
				file.write("\t</meshlight>\n")
			elif exitportal:
				file.write("\t</exit_portal>\n")
			else:
				file.write("\t</model>\n\n")
	elif (objectdup):   # (objecttype == "Empty"):
		if (obj.enableDupGroup):
			group = obj.DupGroup
			groupname = group.name
			print "processing GroupObject \"%s\" (Group \"%s\")..." %(objectname, groupname)
		else:
			print "processing DupObject \"%s\"..." %(objectname)
		for o, m in obj.DupObjects:
			file.write(exportObject(o, meshlist, matlist, file))
		if (obj.enableDupGroup):
			print "end processing GroupObject \"%s\"" %(objectname)
		else:
			print "end processing DupObject \"%s\"" %(objectname)
	else:
		print "Skip \"%s\" (type \"%s\")" %(obj.getName(), obj.getType())
	#return "".join(xml)



#def exportObjects(objlist, meshlist, matlist, filename):
def exportObjects(objlist, meshlist, matlist, file):
	xml = []
	for obj in  objlist:
		#xml.append(exportObject(obj, obj.getMatrix(), meshlist, matlist, filename))
		exportObject(obj, meshlist, matlist, file)
#	return "".join(xml)
#end exportObjects

def getMeshFromObject(ob, container_mesh=None, apply_modifiers=True, vgroups=True, scn=None):
	'''
	ob - the object that you want to get the mesh from
	container_mesh - a Blender.Mesh type mesh that is reused to avoid a new datablock per call to getMeshFromObject
	apply_modifiers - if enabled, subsurf bones etc. will be applied to the returned mesh. disable to get a copy of the mesh.
	vgroup - For mesh objects only, apply the vgroup to the the copied mesh. (slower)
	scn - Scene type. avoids getting the current scene each time getMeshFromObject is called.
	
	Returns Mesh or None
	'''
	
	if not scn:
		scn= BPyMesh.bpy.data.scenes.active
	if not container_mesh:
		#mesh = BPyMesh.bpy.data.meshes.new(ob.name)	
		mesh=Blender.Mesh.New(ob.name)
	else:
		mesh= container_mesh
		mesh.verts= None
	
	ob_type = ob.type
	dataname = ob.getData(1)
	tempob= None
	if apply_modifiers or ob_type != 'Mesh':
		try:
			mesh.getFromObject(ob,0,1)
		except:
			return None
	
	else:
		'''
		Dont apply modifiers, copy the mesh. 
		So we can transform the data. its easiest just to get a copy of the mesh. 
		'''
		tempob= scn.objects.new(ob.getData(mesh=1))
		mesh.getFromObject(tempob, render=1)
		scn.objects.unlink(tempob)

	'''
	if ob_type == 'Mesh':
		if vgroups:
			if tempob==None:
				tempob= Blender.Object.New('Mesh')
			
			tempob.link(mesh)
			try:
				# Copy the influences if possible.
				groupNames, vWeightDict= meshWeight2Dict(ob.getData(mesh=1))
				BPyMesh.dict2MeshWeight(mesh, groupNames, vWeightDict)
			except:
				# if the modifier changes the vert count then it messes it up for us.
				pass
	'''
	
	return mesh

### EXPORT TO FILES ####
def exportIndigoScene(filename, objlist, meshlist, matlist, includelist):
	def build_lists():
		for obj in objlist:
			matrix=obj.getMatrix()
			objectname = obj.getName()
			objecttype = obj.getType()
			objectdup = obj.enableDupGroup or obj.enableDupVerts
			print("Object %s\nType %s\nDup %d"%(objectname, objecttype,objectdup))
			if (objecttype == "Mesh")  or (objecttype == "Curve") or (objecttype == "Text"):
				print("Getting Mesh from Object %s"%objectname)
				meshname=obj.getData(name_only=1)
				if(objecttype == "Curve"):
					meshname +='_cu'

				mesh=getMeshFromObject(obj)
				

				if len(mesh.faces) <= 0:
					print "Skip Object"
					continue
				print("Faces %d"%len(mesh.faces))
				print("Processing %s Mesh"%meshname)
				if (objecttype == "Curve"):
					mesh.materials = obj.getData().getMaterials()
				if (objecttype == "Text"):
					curveName=obj.getData().getName()
					mesh.materials = Blender.Curve.Get(curveName).getMaterials()
				if len(mesh.materials) > 0:
					mat_idx=0
					for mat_i in mesh.materials:
						if mat_i == None:
							continue
						print("material %s found"%mat_i)
						if mat_i != None and matIsLight(mat_i):
							if mat_idx > 0:
								print("Multiple material meshlight!")
								#raise("Error: Multiple material meshlight not allowed!\nMeshlight: %s"%mesh.name)
							try:
								matlist.index(mat_i.name)
							except ValueError:
								print("Appending Emitter %s for export"%mat_i.name)
								matlist.append(mat_i.name)
						#elif mat_i != None and not matIsExitPortal(mat_i) and not matIsLight(mat_i):
						else:
							try:
								matlist.index(mat_i.name)
							except ValueError:
								print("Appending material %s for export\n"%mat_i.name)
								matlist.append(mat_i.name)
						mat_idx=mat_idx+1
				#if len(mesh.faces) <= 0:
				#	print 'WARNING: ignoring object "%s", because linked mesh "%s" has no faces'%(objectname, meshname)
				#	continue
				imeshname = meshname.replace("<","-").replace(">","-")
				pos = matrix.translationPart()
				scl = matrix.scalePart()
				# the blender euler rotation is wrong in some cases and don't match indigo rotation so here a new one:
				rot = Blender.Mathutils.Matrix(matrix[0]/scl[0],matrix[1]/scl[1],matrix[2]/scl[2])
				if objectname.find('.plane') == -1 and objectname.find('.sphere') == -1:
					mat = None
					if (abs(scl[0]-scl[1])>1e-4) or (abs(scl[0]-scl[2])>1e-4):
						print 'WARNING: object "%s" has a nonuniform scaling (x:%f, y:%f, z:%f)'%(objectname, scl[0], scl[1], scl[2])
						mat = Blender.Mathutils.Matrix()
						for v in range(3):
							for z in range(3):
								mat[z][v] = matrix[z][v]
						rot = Blender.Mathutils.Matrix()
						scalefact = 1.0
					else:
						scalefact = (scl[0] + scl[1] + scl[2]) / 3
					flag=0
					if len(meshlist) > 0:
						for m in meshlist:
							if m[0] == imeshname:
								flag=1
								#print "FOUND!"
								break
					if not flag or len(meshlist) == 0:
						#print("Flag %d"%flag)
						meshlist.append([imeshname, mesh, mat])
						print("Appending mesh %s for export"%imeshname)
	#End buils_lists
				

	ws = WorldScale.val*(10**WorldScaleExp.val)
	file = open(filename, 'w')
	file.write("<?xml version=\"1.0\" encoding=\"utf-8\"?>\n\n")
	file.write('<scene>\n')

	# write render_settings
	file.write("\t<renderer_settings>\n")
	file.write("\t\t<width>%d</width>\n" % (SizeX.val*ScaleSize.val/100))
	file.write("\t\t<height>%d</height>\n" % (SizeY.val*ScaleSize.val/100))
	file.write("\t\t<metropolis>%s</metropolis>\n" %(IndigoBoolean(MLT.val)))
	file.write("\t\t<max_num_consec_rejections>%d</max_num_consec_rejections>\n" % MaxNumConsRej.val)
	file.write("\t\t<bidirectional>%s</bidirectional>\n" %(IndigoBoolean(Bidirectional.val)))
	file.write("\t\t<max_depth>%s</max_depth>\n" % MaxDepth)
	file.write("\t\t<logging>%s</logging>\n" %(IndigoBoolean(Logging.val)))
	file.write("\t\t<save_untonemapped_exr>%s</save_untonemapped_exr>\n" %(IndigoBoolean(SaveUTMExr.val)))
	file.write("\t\t<save_tonemapped_exr>%s</save_tonemapped_exr>\n" %(IndigoBoolean(SaveTMExr.val)))
	file.write("\t\t<save_igi>%s</save_igi>\n" %(IndigoBoolean(SaveIgi.val)))
	file.write("\t\t<display_period>%s</display_period>\n" % float(DisplayPeriod.val))
	file.write("\t\t<halt_time>%s</halt_time>\n" % (max(HaltTime.val, -1)))
	file.write("\t\t<halt_samples_per_pixel>%s</halt_samples_per_pixel>\n" % HaltSamples.val)
	file.write("\t\t<frame_upload_period>%s</frame_upload_period>\n" %(FrameUp.val))
	file.write("\t\t<image_save_period>%s</image_save_period>\n" %(ImageSave.val))
	file.write("\t\t<super_sample_factor>%s</super_sample_factor>\n" %(SuperSample.val))
	file.write("\t\t<hybrid>%s</hybrid>\n" %(IndigoBoolean(Hybrid.val)))
	file.write("\t\t<bih_tri_threshold>%s</bih_tri_threshold>\n" %(BIH.val))
	file.write("\t\t<ray_origin_nudge_distance>%f</ray_origin_nudge_distance>\n"%RayNudge.val)
	file.write("\t\t<cache_trees>%s</cache_trees>\n" %(IndigoBoolean(CacheTree.val)))
	file.write("\t\t<num_threads>%d</num_threads>\n"%Threads.val)
	file.write("\t\t<auto_choose_num_threads>false</auto_choose_num_threads>\n")
	file.write("\t\t<aperture_diffraction>%s</aperture_diffraction>\n"%(IndigoBoolean(ApertureDiffr.val)));
	if ApertureDiffr.val:
		file.write("\t\t<post_process_diffraction>%s</post_process_diffraction>\n"%(IndigoBoolean(AperturePost.val)));
	file.write("\t\t<watermark>%s</watermark>\n"%(IndigoBoolean(Watermark.val)));
	if Scene.GetCurrent().getRenderingContext().borderRender:
		print("Enable Border Render")
		region=Scene.GetCurrent().getRenderingContext().border
		print("Region %d %d %d %d"%((SizeX.val*ScaleSize.val/100)*region[0],(SizeY.val*ScaleSize.val/100)*region[1],(SizeX.val*ScaleSize.val/100)*region[2],(SizeY.val*ScaleSize.val/100)*region[3]))
		file.write("\t\t<render_region>\n")
		file.write("\t\t\t<x1>%d</x1>\n"%((SizeX.val*ScaleSize.val/100)*region[0]))
		file.write("\t\t\t<y1>%d</y1>\n"%((SizeY.val*ScaleSize.val/100)-(SizeY.val*ScaleSize.val/100)*region[3]))
		file.write("\t\t\t<x2>%d</x2>\n"%((SizeX.val*ScaleSize.val/100)*region[2]))
		file.write("\t\t\t<y2>%d</y2>\n"%((SizeY.val*ScaleSize.val/100)-(SizeY.val*ScaleSize.val/100)*region[1]))
		file.write("\t\t</render_region>\n")
	if AlphaMask.val:
		file.write("\t\t<render_foreground_alpha>true</render_foreground_alpha>\n")
	file.write("\t\t<splat_filter>\n")
	if SplatFilter.val == 0:
		file.write("\t\t\t<box/>\n")
	if SplatFilter.val == 1:
		file.write("\t\t\t<gaussian></gaussian>\n")
	if SplatFilter.val == 2:
		file.write("\t\t\t<mn_cubic>\n")
		file.write("\t\t\t\t<blur>%f</blur>\n"%SplatF_MN_Blur.val)
		file.write("\t\t\t\t<ring>%f</ring>\n"%SplatF_MN_Ring.val)
		file.write("\t\t\t</mn_cubic>\n")
	file.write("\t\t</splat_filter>\n")
	file.write("\t\t<downsize_filter>\n")
	if DownsizeFilter.val == 0:
		file.write("\t\t\t<box/>\n")
	if DownsizeFilter.val == 1:
		file.write("\t\t\t<gaussian></gaussian>\n")
	if DownsizeFilter.val == 2:
		file.write("\t\t\t<mn_cubic>\n")
		file.write("\t\t\t\t<blur>%f</blur>\n"%DownsizeF_MN_Blur.val)
		file.write("\t\t\t\t<ring>%f</ring>\n"%DownsizeF_MN_Ring.val)
		file.write("\t\t\t</mn_cubic>\n")
	file.write("\t\t</downsize_filter>\n")
	file.write("\t</renderer_settings>\n\n")

	# write tone mapping		
	if ToneMapType.val == 0:
		#TODO
		if (Autoexp.val == 0):
			exposure = ToneMapScale.val * (10**ToneMapScaleExp.val)
		else:
#			expfact = 0.2
#			exposure = ((1/float(Shutter.val))*expfact)*(float(FilmIso.val)/100)
			if Shutter.val == 0:
				print "WARNING: Shutter is not set (using 125)"
				Shutter.val = 125
			exposure = ToneMapScale.val * (10**ToneMapScaleExp.val) * float(FilmIso.val) / float(Shutter.val)
		file.write("\t<tonemapping>\n")
		file.write("\t\t<linear>\n")
		file.write("\t\t\t<scale>%g</scale>\n" %exposure)
		file.write("\t\t</linear>\n")
		file.write("\t</tonemapping>\n\n")
	if ToneMapType.val == 1:
		file.write("\t<tonemapping>\n")
		file.write("\t\t<reinhard>\n")
		file.write("\t\t\t<pre_scale>%g</pre_scale>\n" %ToneMapPreScale.val)
		file.write("\t\t\t<post_scale>%g</post_scale>\n" %ToneMapPostScale.val)
		file.write("\t\t\t<burn>%g</burn>\n" %ToneMapBurn.val)
		file.write("\t\t</reinhard>\n")
		file.write("\t</tonemapping>\n\n")
	if ToneMapType.val == 2:
		file.write("\t<tonemapping>\n")
		file.write("\t\t<camera>\n");
		file.write("\t\t<ev_adjust>%g</ev_adjust>\n"%ToneMapEV.val)
		file.write("\t\t<film_iso>%g</film_iso>\n"%float(FilmIso.val))
		file.write("\t\t<response_function_path>data/camera_response_functions/dscs315.txt</response_function_path>\n")
		file.write("\t\t</camera>\n")
		file.write("\t</tonemapping>\n\n")
	# write Color background
	if EnvType.val == 0:
		worldcolor = Blender.World.Get('World').getHor()
		file.write("\t<background>\n")
		file.write("\t\t<spectrum>\n")
		file.write("\t\t\t<rgb>\n")
		file.write("\t\t\t\t<rgb>%g %g %g</rgb>\n" %(BGColor.val[0]*BGGain.val, BGColor.val[1]*BGGain.val, BGColor.val[2]*BGGain.val))
		file.write("\t\t\t\t<gamma>2.2</gamma>\n")
		file.write("\t\t\t</rgb>\n")
		file.write("\t\t</spectrum>\n")
		file.write("\t</background>\n\n")
				
	##### Process sun ######
	if EnvType.val == 1:
		for obj in  objlist:
			if obj.getType() == "Lamp":
				if obj.data.getType() == 1: # sun object
					print("Processing sun lamp object...\n")
					invmatrix = Mathutils.Matrix(obj.getInverseMatrix())
					file.write("\t<skylight>\n")
					file.write("\t\t<sundir>%f %f %f</sundir>\n" % (invmatrix[0][2], invmatrix[1][2], invmatrix[2][2]))
					file.write("\t\t<turbidity>%f</turbidity>\n" %Turbidity.val)
					if ExtraAtm.val:
						file.write("\t\t<extra_atmospheric>%s</extra_atmospheric>\n" %IndigoBoolean(ExtraAtm.val))
					file.write("\t</skylight>\n\n")
			
	##### Process Env Map #####
	if EnvType.val == 2:
		if EnvFile.val != "none" and EnvFile.val != "":
			file.write("\t<env_map>\n")
			file.write("\t\t<%s>\n" %(EnvMapTypeV[EnvMapType.val]))
#			file.write("\t\t\t<path>HDR\%s</path>\n" %(EnvFile.val))
			file.write("\t\t\t<path>%s</path>\n" %(relpath(file.name, EnvFile.val)))
			file.write("\t\t\t<gain>%g</gain>\n" %(EnvGain.val*1e4)) #SmartDen: scaled for using with meshlight
			if EnvMapType.val == 0:
				file.write("\t\t\t<width>%d</width>\n" %(EnvWidth.val))
			file.write("\t\t</%s>\n" %(EnvMapTypeV[EnvMapType.val]))
			file.write("\t</env_map>\n\n")

	##### Process camera ######
	currentscene = Scene.GetCurrent()
	camObj = currentscene.getCurrentCamera()
	if camObj:
		print "processing Camera..."
		matrix = camObj.getMatrix()
		pos = matrix[3]
		forwards = matrix[2]
		up = matrix[1]
		file.write("\t<camera>\n")
		file.write("\t\t<pos>%f %f %f</pos>\n" % (pos[0]*ws, pos[1]*ws, pos[2]*ws))
		file.write("\t\t<up>%f %f %f</up>\n" % (up[0], up[1], up[2]))
		file.write("\t\t<forwards>%f %f %f</forwards>\n" % (-forwards[0], -forwards[1], -forwards[2]))
		aspect = float(SizeX.val) / float(SizeY.val)
		film = 0.001*FilmWidth.val
	
		f_stop = float(FStop.val) * 0.1
		FOV=camObj.data.angle
		if aspect < 1:
			FOV = FOV*aspect
		lens_sensor_dist = film/(2*math.tan(FOV*math.pi/360))
		#Z=(film/aspect)/1500
		#Dh=lens_sensor_dist*lens_sensor_dist/(f_stop*Z)
		#Dn=FocusDistance.val*Dh/(Dh+(FocusDistance.val-f))

		if FStop.val == 0:
			print "WARNING: FStop is not set (using 8.0)"
			FStop.val = 80
		aperture_radius = lens_sensor_dist / (2*f_stop)
		print "  Film Width %f"%film
		print "  FOV: %f"%FOV
		print "  lens-sensor-distance: %f m"%lens_sensor_dist
		print "  aperture-radius: %f m"%aperture_radius
		file.write("\t\t<aperture_radius>%f</aperture_radius>\n" % aperture_radius)
		file.write("\t\t<exposure_duration>%f</exposure_duration>\n"%float(1/float(Shutter.val)))
		file.write("\t\t<focus_distance>%f</focus_distance>\n" % FocusDistance.val)
		file.write("\t\t<aspect_ratio>%g</aspect_ratio>\n" % aspect)
		file.write("\t\t<sensor_width>%f</sensor_width>\n" % film)
		file.write("\t\t<lens_sensor_dist>%f</lens_sensor_dist>\n" % lens_sensor_dist)
		file.write("\t\t<white_balance>%s</white_balance>\n" % whiteBalanceV[WhiteBalance.val])
#		if ApertureDiffr.val:
		file.write("\t\t<aperture_shape>\n")
		if ApertureType.val == 1:
			file.write("\t\t\t<circular></circular>\n")
		elif ApertureType.val == 2:
			file.write("\t\t\t<generated>\n")
			file.write("\t\t\t\t<num_blades>%d</num_blades>\n"%BladesNum.val)
			file.write("\t\t\t\t<blade_offset>%f</blade_offset>\n"%float(BladeOffset.val))
			file.write("\t\t\t\t<blade_curvature_radius>%f</blade_curvature_radius>\n"%float(BladeCurvature.val))
			file.write("\t\t\t\t<start_angle>%f</start_angle>\n"%float(StartAngle.val))
			file.write("\t\t\t</generated>\n")
		file.write("\t\t</aperture_shape>\n")
		if not ObstacleMap.val == '':
			file.write("\t\t<obstacle_map>\n")
			file.write("\t\t\t<path>%s</path>\n"%(relpath(indigopath, ObstacleMap.val)))
			file.write("\t\t</obstacle_map>\n")

		if AutoFocus.val==1:
			file.write("\t\t<autofocus/>\n")
		#if Bloom.val==1:
		#	file.write("\t\t<bloom>\n")
		#	file.write("\t\t<weight>%3f</weight>\n" %BloomW.val)
		#	file.write("\t\t<radius>%3f</radius>\n" %BloomR.val)
		#	file.write("\t\t</bloom>\n")
		#if Glare.val==1:                                                     #BF
		#	file.write("\t\t<glare>\n")
		#	file.write("\t\t<num_blades>%3f</num_blades>\n" %GlareB.val)
		#	file.write("\t\t<weight>%3f</weight>\n" %GlareW.val)
		#	file.write("\t\t<radius>%3f</radius>\n" %GlareR.val)
		#	file.write("\t\t</glare>\n")
		file.write("\t</camera>\n\n")

	#	for i in includelist:
	#		file.write("\t<include>\n")
	#		file.write("\t\t<pathname>%s</pathname>\n"%i)
	#		file.write("\t</include>\n\n")

	print("build_lists")
	build_lists()
	#for a in meshlist:
	#	print a

	if (SepMatFile.val):
		matsfn = Blender.sys.makename(filename, '-mats.xml')
		print("Exporting %d Materials\n"%len(matlist))
		matsfile=open(matsfn,'w')
		matsfile.write("<?xml version=\"1.0\" encoding=\"utf-8\"?>\n\n")
		matsfile.write('<scenedata>\n')
		matsfile.write("\t<material>\n")
		matsfile.write("\t\t<name>Default</name>\n")
		matsfile.write("\t\t<diffuse>\n")
		matsfile.write("\t\t\t<albedo_spectrum>\n\t\t\t\t<rgb>\n\t\t\t\t\t<rgb>1 1 1</rgb>\n\t\t\t\t\t<gamma>2.2</gamma>\n\t\t\t\t</rgb>\n\t\t\t</albedo_spectrum>\n")
		matsfile.write("\t\t</diffuse>\n")
		matsfile.write("\t</material>\n\n")
		matsfile.write("\t<material>\n")
		matsfile.write("\t\t<name>null</name>\n")
		matsfile.write("\t\t<null_material/>\n")
		matsfile.write("\t</material>\n\n")
		exportedmatlist = ["Default", "null"]
		includelist = []
		while len(matlist) > 0:
			matname = matlist[0]
			mat = Blender.Material.Get(matname)
			matlist.remove(matname)
			try:
				exportedmatlist.index(matname)
			except:
				print("Exporting material %s"%mat)
				exportMaterial(mat, matsfile, exportedmatlist, includelist)
		if len(includelist) > 0:
			for i in includelist:
				matsfile.write("\t<include>\n")
				matsfile.write("\t\t<pathname>%s</pathname>\n"%i)
				matsfile.write("\t</include>\n\n")
		matsfile.write('</scenedata>\n')
		matsfile.close

		relmatsfn = relpath(filename, matsfile.name)		
		file.write("\t<include>\n")
		file.write("\t\t<pathname>%s</pathname>\n"%relmatsfn)
		file.write("\t</include>\n\n")
		#include.append(relmatsfn)
	else:
		file.write("\t<material>\n")
		file.write("\t\t<name>Default</name>\n")
		file.write("\t\t<diffuse>\n")
		file.write("\t\t\t<albedo_spectrum>\n\t\t\t\t<rgb>\n\t\t\t\t\t<rgb>1 1 1</rgb>\n\t\t\t\t\t<gamma>2.2</gamma>\n\t\t\t\t</rgb>\n\t\t\t</albedo_spectrum>\n")
		file.write("\t\t</diffuse>\n")
		file.write("\t</material>\n\n")
		file.write("\t<material>\n")
		file.write("\t\t<name>null</name>\n")
		file.write("\t\t<null_material/>\n")
		file.write("\t</material>\n\n")
		exportedmatlist = ["Default", "null"]
		includelist = []
		while len(matlist) > 0:
			matname = matlist[0]
			mat = Blender.Material.Get(matname)
			matlist.remove(matname)
			try:
				exportedmatlist.index(matname)
			except:
				print("Exporting material %s"%mat)
				exportMaterial(mat, file, exportedmatlist)
		if len(includelist) > 0:
			for i in includelist:
				matsfile.write("\t<include>\n")
				matsfile.write("\t\t<pathname>%s</pathname>\n"%i)
				matsfile.write("\t</include>\n\n")
		
		#relmatsfn = relpath(filename, file.name)		
		#file.write("\t<include>\n")
		#file.write("\t\t<pathname>%s</pathname>\n"%relmatsfn)
		#file.write("\t</include>\n\n")
		#include.append(relmatsfn)


	print("Found %d Meshes"%len(meshlist))
	if (SepMeshFile.val) and (ExportMeshs.val):#write Meshs into separate file
		print("Exporting in separate Meshs file")
		meshsfn = Blender.sys.makename(filename, '-meshs.xml')
		meshsfile=open(meshsfn,"w")
		meshsfile.write("<?xml version=\"1.0\" encoding=\"utf-8\"?>\n\n")
		meshsfile.write('<scenedata>\n')
		for mb in meshlist:
			exportMesh(mb, matlist, meshsfile)
		meshsfile.write('</scenedata>\n')
		meshsfile.close()
		meshsfile.close()
		relmeshsfn = relpath(filename, meshsfn)
		file.write("\t<include>\n")
		file.write("\t\t<pathname>%s</pathname>\n"%relmeshsfn)
		file.write("\t</include>\n\n")
	elif SepMeshFile.val:
		print("Including Meshs file")
		meshsfn = Blender.sys.makename(filename, '-meshs.xml')
		relmeshsfn = relpath(filename, meshsfn)
		file.write("\t<include>\n")
		file.write("\t\t<pathname>%s</pathname>\n"%relmeshsfn)
		file.write("\t</include>\n\n")
	else:
		print("Exporting Mesh in Scene file")
		for mb in meshlist:
			exportMesh(mb, matlist, file)


	print("exportObjects")
	exportObjects(objlist, meshlist, matlist, file)
	#file.write(objsxml)

	##### Check for ground plane ######
	if GroundPlane.val is 1:
		file.write("\t<plane>\n")
		file.write("\t\t<normal>0 0 1</normal>\n")
		file.write("\t\t<dist>-0.000001</dist>\n")
		file.write("\t\t<material_name>Default</material_name>\n")
		file.write("\t</plane>\n\n")

	file.write('</scene>\n')
	file.close()


def exportMatPreview(mat):
	global indigopath
	print("\nINDIGO MATERIAL PREVIEW EXPORT\n")
	time1 = Blender.sys.time()
	filename = os.path.join(indigopath, "previews", matPreviewFile)
	filepath = Blender.sys.dirname(filename)
	matsfn = os.path.join(indigopath, "previews", "previewmats.xml") # DaveC
	file=open(matsfn,"w")
	relmatsfn = relpath(indigopath, matsfn)
	includelist = []
	if not(os.path.exists(filename)): # if not found try alternative old ".xml" file extension
		print 'no preview file found "%s"'%(filename)
	if os.path.exists(filename):
		# Define some dummy materials
		file.write("<?xml version=\"1.0\" encoding=\"utf-8\"?>\n\n")
		file.write('<scenedata>\n')
		file.write("\t<material>\n")
		file.write("\t\t<name>default</name>\n")
		file.write("\t\t<diffuse>\n")
		file.write("\t\t\t<albedo_spectrum>\n\t\t\t\t<rgb>\n\t\t\t\t\t<rgb>0.8 0.8 0.8</rgb>\n\t\t\t\t\t<gamma>2.2</gamma>\n\t\t\t\t</rgb>\n\t\t\t</albedo_spectrum>\n")
		file.write("\t\t</diffuse>\n")
		file.write("\t</material>\n\n")
		file.write("\t<material>\n")
		file.write("\t\t<name>null</name>\n")
		file.write("\t\t<null_material/>\n")
		file.write("\t</material>\n\n")
		exportedmatlist = ["Default", "null"]
		#matsxml.append(exportMaterial(mat, filepath, exportedmatlist, includelist, PreviewName))  #BF
		exportMaterial(mat,file, exportedmatlist, includelist, PreviewName)
		file.write("</scenedata>");
		time2 = Blender.sys.time()
		print("Processing time: %f\n" %(time2-time1))
		launchIndigo(filename)
	else:
		print 'also no old material preview file found "%s", sorry'%(filename)



def export(filename, anim):
	global IndigoFilePath
	IndigoFilePath=filename
	print("\nINDIGO EXPORT\n")
	time1 = Blender.sys.time()
# Following lines were replaced by Blender.sys functions below
#	filepath = os.path.dirname(filename)
#	(filebase, fileext) = os.path.splitext(os.path.basename(filename))
#	meshsfn = os.path.join(filepath, filebase+'-meshs', fileext)
#	matsfn = os.path.join(filepath, filebase+'-mats', fileext)
	filepath = Blender.sys.dirname(filename)
	
	objlist = []
	matlist = []
	meshlist = []
	Draw.Draw()#Force an immediate redraw

	if anim:
		startF = Blender.Get('staframe')
		endF = Blender.Get('endframe')
		print('start %d end %d'%(startF,endF))
		for i in range(startF, endF+1):
			print('exporting frame %d\n'%i)
			Blender.Set('curframe', i)
			Blender.Redraw()
#			#get all the objects in this scene
			activelayers = Window.ViewLayers()
			for obj in Blender.Scene.GetCurrent().getChildren():
				visible = 0
				for layer in obj.layers:
					if layer in activelayers:
						print('Object %s in layer %d'%(obj.getName(),layer))
						visible = 1
				if visible:
					objlist.append(obj)
			framefilename = makename(filename, "-"+str(i)+".igs")
			objsxml = exportObjects(objlist, meshlist, matlist, filename)
			if(ExportMeshs.val) and not (SepMeshFile.val):#write Meshs into Scene file
				meshsxml = exportMeshs(meshlist, matlist)
				exportIndigoScene(framefilename, objlist, [meshsxml,objsxml], [relmatsfn])
			else:
				exportIndigoScene(framefilename, objlist, [objsxml], [relmatsfn,relmeshsfn])
			del objlist[:]#delete all list items

			if(ExportMeshs.val) and not (SepMeshFile.val):
				del meshlist[:]
		
		Blender.Set('curframe', endF)
		Blender.Redraw()
		if(ExportMeshs.val) and (SepMeshFile.val):
			meshsxml = exportMeshs(meshlist, matlist)
			exportIndigoScenedata(meshsfn, [meshsxml], [])
		includelist = []
		exportMaterial(matlist, filepath,includelist)
		exportIndigoScenedata(matsfn, [matsxml], [])
	else:
		#get all the objects in this scene in visible layers
		activelayers = Window.ViewLayers()
		for obj in Blender.Scene.GetCurrent().objects:
			visible = 0
			for layer in obj.layers:
				if layer in activelayers:
					print("Object %s in layer %d"%(obj.getName(),layer))
					visible = 1
			if visible:
				objlist.append(obj)
		include = []
		#objsxml = exportObjects(objlist, meshlist, matlist,filename)
		#print("Exporting %d Objects, %d Meshes and %d Materials\n"%(len(objlist),len(meshlist),len(matlist)))
		exportIndigoScene(filename, objlist, meshlist, matlist, include)
		if ExportMeshs.val:
			exportMeshs_flag=1
		else:
			meshsxml = ''
			#include.append(relmeshsfn)
			matlist = []
			for m in Material.Get():
				matlist.append(m.getName())
			print "mesh export is disabled - exporting all available materials"
		
		#exportIndigoScene(filename, objlist, meshlist, matlist, include)
	time2 = Blender.sys.time()
	print("Processing time: %f\n" %(time2-time1))
	if not(anim) and (ExecuteIndigo.val == 1):
		launchIndigo(filename)
	if (anim) and (ExecuteIndigo.val == 1):
		print("Exporting Animation")
		launchAnimIndigo(filename)

def exportStill(filename):
	print("Export Still to %s"%filename)
	export(filename, 0)

def exportAnim(filename):
	export(filename, 1)





print ""
print "----------------------"
print "BLENDIGO v1.0.8"
print "----------------------"
# print "rel-path %s"%(relpath("c:\\indigo\\", "C:\\myxxx\\this.jpg"))

#TODO
#[16:42]	<OnoSendai>	You should use forwards slashes for path separators
#[16:42]	<OnoSendai>	e.g. for nkdata
#[16:43]	<OnoSendai>	and u need to make sure all filenames are case sensitive
#[16:43]	<OnoSendai>	e.g. for nkdata

linux = 0
if sys.platform[:3] == 'win':
	print "  Windows-platform: "+sys.platform+"   os.sep: "+os.sep
else:
	linux = 1
	print "  Linux-platform: "+sys.platform+"  os.sep: "+os.sep
# get blenders 'bpydata' directory to locate 'IndigoWrapper.conf'
iwfilename = os.path.join(Blender.Get("datadir"), 'IndigoWrapper.conf')
if os.path.exists(iwfilename):
	f = open(iwfilename,'r')
	indigopath=f.readline().strip()
	f.close()
	if indigopath[-1:] != os.sep:
		indigopath = indigopath + os.sep
	if os.path.exists(os.path.join(indigopath,"indigo.exe")):
		indigofile=os.path.join(indigopath,"indigo.exe")
		print "  indigo path: %s"%indigofile
	elif os.path.exists(os.path.join(indigopath,"indigo_console")):
		indigofile=os.path.join(indigopath,"indigo_console")
		print "  indigo path: %s"%indigofile
	else:
		indigofile=os.path.join(indigopath,"indigo.exe")
		print "can't locate indigo path \""+indigofile+"\"! please correct your \""+iwfilename+"\" file."
		Draw.PupMenu("can't find Indigo at \""+indigofile+"\"! please check installation%t|OK%x1")
else:
	print "can't find file \""+iwfilename+"\"! this file must contain the path to the Indigo directory."
	Draw.PupMenu("can't find file \""+iwfilename+"\"! please check installation%t|OK%x1")




######################################################
# Settings GUI
######################################################


###psor's add
ExecuteIndigo = Draw.Create(1)
DefaultExport = Draw.Create(1)
###END


# Assign event numbers to buttons
evtNoEvt	= 0
evtExport   = 1
evtExportAnim   = 2
openCamera = 6
openEnv = 7
openRSet = 8
openSSet = 9
openTmap = 10
evtloadimg = 11
evtchangesize = 12
evtIsNumber = 3
evtFocusS = 4
evtFocusC = 5
evtHStop = 13 #BF
openMat = 14 #SmartDen
evtloadobstacle = 15

# Set initial values of buttons

##  <size>800 600</size>

sceneSizeX = Scene.GetCurrent().getRenderingContext().imageSizeX()
sceneSizeY = Scene.GetCurrent().getRenderingContext().imageSizeY()

SizeX = Draw.Create(sceneSizeX)
SizeY = Draw.Create(sceneSizeY)

strScaleSize = "Render scale:%t | 200 % %x200 | 150 % %x150 | 100 % %x100 | 75 % %x75 | 50 % %x50 | 25 % %x25 | 20 % %x20 | 10 % %x10 "
ScaleSize = Draw.Create(100)

ColExponent = Draw.Create(2.2)

AlphaMask=Draw.Create(0)

SplatFilter=Draw.Create(2)
SplatF_MN_Blur=Draw.Create(0.3)
SplatF_MN_Ring=Draw.Create(0.3)
DownsizeFilter=Draw.Create(2)
DownsizeF_MN_Blur=Draw.Create(0.3)
DownsizeF_MN_Ring=Draw.Create(0.3)

##  <metropolis>1</metropolis>
MLT = Draw.Create(1)

##  <large_mutation_prob>0.1</large_mutation_prob>
LMP = Draw.Create(0.4)

##  <max_change>0.02</max_change>
MaxChange = Draw.Create(0.02)

##  <max_num_consec_rejections>100</max_num_consec_rejections>
MaxNumConsRej = Draw.Create(1000)

##  <russian_roulette_live_prob>0.7</russian_roulette_live_prob>
#RRLP = Draw.Create(0.7) #BF

##  <max_depth>1000</max_depth>
MaxDepth = Draw.Create(1000)

##  <bidirectional>true</bidirectional>
Bidirectional = Draw.Create(1)

##  <strata_width>14</strata_width>
#StrataWidth = Draw.Create(14) #BF

##Threads
Threads = Draw.Create(1)

##  <logging>0</logging>
Logging = Draw.Create(1)

##  <save_untonemapped_exr>false</save_untonemapped_exr>
SaveUTMExr = Draw.Create(0)

##  <save_tonemapped_exr>false</save_tonemapped_exr>
SaveTMExr = Draw.Create(0)

##  <save_igi>false</save_igi>
SaveIgi = Draw.Create(0)

UseGUI = Draw.Create(1)

Watermark = Draw.Create(1)

##  <display_period>20</display_period>
DisplayPeriod = Draw.Create(20)

#HaltTime
HaltTime = Draw.Create(-1)

#Halt Samples
HaltSamples = Draw.Create(-1)

#Frameup
FrameUp = Draw.Create(20)

#ImageSave
ImageSave = Draw.Create(120)

##FilmIso
FilmIso = Draw.Create(100)
Shutter = Draw.Create(125)    
##  <sensor_width>0.036</sensor_width>
FilmWidth = Draw.Create(36)
##  <lens_radius>0.0</lens_radius>
FStop = Draw.Create(80)
##  <focus_distance>2.0</focus_distance>
FocusDistance = Draw.Create(2.0)


AutoFocus = Draw.Create(1)

ApertureDiffr = Draw.Create(1)
AperturePost=Draw.Create(1)
ApertureType = Draw.Create(1)
BladesNum = Draw.Create(6)
BladeOffset = Draw.Create(0.4)
BladeCurvature = Draw.Create(3)
StartAngle = Draw.Create(0.2)
ObstacleMap = Draw.Create('')

#Bloom = Draw.Create(0)
#BloomW = Draw.Create(0.1)
#BloomR = Draw.Create(0.05)

#Glare = Draw.Create(0)      #BF
#GlareW = Draw.Create(0.1)   #BF
#GlareR = Draw.Create(0.05)  #BF
#GlareB = Draw.Create(5.0)   #BF

##  <white_balance>D65</white_balance>
#strWhiteBalance = "White Balance:%t | D50 %x50 | D55 %x55 | D65 %x65 | D75 %x75"
strWhiteBalance = "White Balance:%t | E %x0 | D50 %x1 | D55 %x2 | D65 %x3 | D75 %x4 "
strWhiteBalance += "| A %x5 | B %x6 | C %x7 | 9300 %x8 | F2 %x9 | F7 %x10 | F11 %x11"
WhiteBalance = Draw.Create(0)
whiteBalanceV = {}
whiteBalanceV[0] = "E"
whiteBalanceV[1] = "D50"
whiteBalanceV[2] = "D55"
whiteBalanceV[3] = "D65"
whiteBalanceV[4] = "D75"
whiteBalanceV[5] = "A"
whiteBalanceV[6] = "B"
whiteBalanceV[7] = "C"
whiteBalanceV[8] = "9300"
whiteBalanceV[9] = "F2"
whiteBalanceV[10] = "F7"
whiteBalanceV[11] = "F11"



## Environment Type
strEnvType = "Env Type:%t | Background Colour %x0 | Physical Sky+sun %x1 | Environment Map %x2 | None (lit by mesh emitters) %x3"
EnvType = Draw.Create(1)

##  <turbidity>2.2</turbidity>
Turbidity = Draw.Create(2.2)
ExtraAtm = Draw.Create(0)

##  <sky_gain>2.0</sky_gain>
#SkyGain = Draw.Create(0.005)

GroundPlane = Draw.Create(0)

## Separate materials
SepMatFile = Draw.Create(0)

## Separate meshs
SepMeshFile = Draw.Create(0)

##ExpotMeshs
ExportMeshs = Draw.Create(1)

## Environment map
EnvFile = Draw.Create("none")
strEnvMapType = "Map type:%t | Spherical %x0 | LatLong %x1"
EnvMapType = Draw.Create(1)
EnvMapTypeV = {}
EnvMapTypeV[0] = "spherical"
EnvMapTypeV[1] = "latlong"
EnvGain = Draw.Create(1.0)
EnvWidth = Draw.Create(640)

## Exposure
ToneMapScale = Draw.Create(1.0)
ToneMapScaleExp = Draw.Create(0)
ToneMapEV=Draw.Create(0)


Autoexp = Draw.Create(0)
HStop = Draw.Create(0)

## Tonemapping
strToneMapType = "ToneMap type:%t | Linear %x0 | Reinhard %x1 | Camera %x2"
ToneMapType = Draw.Create(0)
ToneMapPreScale = Draw.Create(3.0)
ToneMapPostScale = Draw.Create(1.0)
ToneMapBurn = Draw.Create(6.0)

## Overall color gain
DiffuseGain = Draw.Create(1.00)
SpecularGain = Draw.Create(1.00)

## Background color
BGColor = Draw.Create(0.0, 0.0, 0.0)
BGGain = Draw.Create(1.0)

# text color fix
textcol = [0, 0, 0]

# World scale
WorldScale = Draw.Create(1.0)
WorldScaleExp = Draw.Create(0)

# Supersample
SuperSample = Draw.Create(2)

# Pathtrace/MLT Hybrid
Hybrid = Draw.Create(0)
BIH = Draw.Create(500000)

RayNudge = Draw.Create(0.0001)

# Cache KD-trees to disk
CacheTree = Draw.Create(1)

# load/save other settings to blender scene
def readWriteSettings(write=0):
	global DefaultExport, ExportMeshs, ScaleSize, ColExponent, FilmWidth
	global FStop, FocusDistance, AutoFocus
	global WhiteBalance, FilmIso, MLT, LMP, MaxChange, MaxNumConsRej
	global SplatFilter,SplatF_MN_Blur, SplatF_MN_Ring, DownsizeFilter,DownsizeF_MN_Blur, DownsizeF_MN_Ring, Bidirectional, MaxDepth, Threads, Logging
	global GroundPlane, SaveUTMExr, SaveTMExr, SaveIgi, USeGUI, DisplayPeriod, ToneMapType
	global ToneMapScale, ToneMapScaleExp, ToneMapEV, Shutter
	global Autoexp, ToneMapPreScale, ToneMapPostScale, ToneMapBurn, HStop, SuperSample, Glare, GlareW, GlareR, GlareB, BIH, RayNudge, Hybrid, CacheTree
	global EnvType, EnvFile, EnvMapType, EnvGain, EnvWidth, Turbidity, ExtraAtm
	global SepMatFile, SepMeshFile, ExecuteIndigo, HaltTime, HaltSamples, FrameUp, ImageSave
	try:
		d = Blender.Scene.GetCurrent().properties['indigo'].convert_to_pyobject()
	except KeyError:
		d = {}
	readWriteValue(d, 'DefaultExport', DefaultExport, write, 1)
	readWriteValue(d, 'ExportMeshs', ExportMeshs, write, 1)
	readWriteValue(d, 'scale size', ScaleSize, write, 100)
	readWriteValue(d, 'colexponent', ColExponent, write, 2.2)
	readWriteValue(d, 'film width', FilmWidth, write, 36)
	readWriteValue(d, 'fstop', FStop, write, 80)
	readWriteValue(d, 'hstop', HStop, write, 0)
	readWriteValue(d, 'focusdistance', FocusDistance, write, 2.0)
	readWriteValue(d, 'autofocus', AutoFocus, write, 1)
	readWriteValue(d, 'aperture_diffraction', ApertureDiffr, write, 0)
	readWriteValue(d, 'post_process_diffraction', AperturePost, write, 1)
	readWriteValue(d, 'aperturen_type', ApertureType, write, 2)
	readWriteValue(d, 'aperture_blades_num', BladesNum, write, 8)
	readWriteValue(d, 'aperture_blade_offset', BladeOffset, write, 0.4)
	readWriteValue(d, 'aperture_blade_curvature', BladeCurvature, write, 3)
	readWriteValue(d, 'aperture_start_angle',StartAngle, write, 0.2)
	readWriteValue(d, 'opstacle_map', ObstacleMap, write, '')
	readWriteValue(d, 'white balance', WhiteBalance, write, 0)
	readWriteValue(d, 'filmiso', FilmIso, write, 100)
	readWriteValue(d, 'mlt', MLT, write, 1)
	readWriteValue(d, 'lmp', LMP, write, 0.4)
	readWriteValue(d, 'splat_filter', SplatFilter,write, 2)
	readWriteValue(d, 'splatf_mn_blur', SplatF_MN_Blur,write, 0.3)
	readWriteValue(d, 'splatf_mn_ring', SplatF_MN_Ring,write, 0.3)
	readWriteValue(d, 'downsize_filter', DownsizeFilter,write, 2)
	readWriteValue(d, 'downsizef_mn_blur', DownsizeF_MN_Blur,write, 0.3)
	readWriteValue(d, 'downsizef_mn_ring', DownsizeF_MN_Ring,write, 0.3)
	readWriteValue(d, 'maxchange', MaxChange, write, 0.02)
	readWriteValue(d, 'maxnumconsrej', MaxNumConsRej, write, 1000)
	readWriteValue(d, 'bidirectional', Bidirectional, write, 1)
	readWriteValue(d, 'maxdepth', MaxDepth, write, 1000)
	readWriteValue(d, 'threads', Threads, write, 1)
	readWriteValue(d, 'logging', Logging, write, 1)
	readWriteValue(d, 'groundplane', GroundPlane, write, 0)
	readWriteValue(d, 'saveutmexr', SaveUTMExr, write, 0)
	readWriteValue(d, 'savetmexr', SaveTMExr, write, 0)
	readWriteValue(d, 'saveigi', SaveIgi, write, 0)
	readWriteValue(d, 'usegui',UseGUI,write,1)
	readWriteValue(d,'watermark',Watermark,write,0)
	readWriteValue(d, 'display period', DisplayPeriod, write, 20)
	readWriteValue(d, 'tonemaptype', ToneMapType, write, 1)
	readWriteValue(d, 'tonemapscale', ToneMapScale, write, 1.0)
	readWriteValue(d, 'tonemapscaleexp', ToneMapScaleExp, write, 0)
	readWriteValue(d, 'tonemapev', ToneMapEV, write, 0)
	readWriteValue(d, 'shutter', Shutter, write, 125)
	readWriteValue(d, 'autoexp', Autoexp, write, 0)
	readWriteValue(d, 'tonemapprescale', ToneMapPreScale, write, 4.0)
	readWriteValue(d, 'tonemappostscale', ToneMapPostScale, write, 1.2)
	readWriteValue(d, 'tonemapburn', ToneMapBurn, write, 6.0)
	readWriteValue(d, 'envtype', EnvType, write, 1)
	readWriteValue(d, 'envfile', EnvFile, write, '')
	readWriteValue(d, 'envmaptype', EnvMapType, write, 1)
	readWriteValue(d, 'envgain', EnvGain, write, 1.0)
	readWriteValue(d, 'envwidth', EnvWidth, write, 640)
	readWriteValue(d, 'turbidity', Turbidity, write, 2.2)
	readWriteValue(d, 'extraAtm', ExtraAtm,write,0)
	readWriteValue(d, 'sepmatfile', SepMatFile, write, 0)
	readWriteValue(d, 'sepmeshfile', SepMeshFile, write, 0)
	readWriteValue(d, 'ExecuteIndigo', ExecuteIndigo, write, 1)
	readWriteValue(d, 'HaltTime', HaltTime, write, -1.0)
	readWriteValue(d, 'HaltSamples', HaltSamples, write, -1.0)
	readWriteValue(d, 'frameup', FrameUp, write, 60)
	readWriteValue(d, 'imagesave', ImageSave, write, 120)
	readWriteValue(d, 'super_sample_factor', SuperSample, write, 2)
	readWriteValue(d, 'bih', BIH, write, 500000)                      #BF
	readWriteValue(d, 'rayNudge',RayNudge, write, 0.0001)
	readWriteValue(d, 'hybrid', Hybrid, write, 0)
	readWriteValue(d, 'cachetrees', CacheTree, write, 1)
	try:
		if write == 0:
			BGColor.val = (d['backgroundcolor.r'], d['backgroundcolor.g'], d['backgroundcolor.b'])
		else:
			d['backgroundcolor.r'] = BGColor.val[0]
			d['backgroundcolor.g'] = BGColor.val[1]
			d['backgroundcolor.b'] = BGColor.val[2]
	except KeyError:
		print ""
	readWriteValue(d, 'backgroundgain', BGGain, write, 1.0)
	readWriteValue(d, 'worldscale', WorldScale, write, 1.0)
	readWriteValue(d,'worldscaleexp',WorldScaleExp,write,0)
	if write != 0:
		Blender.Scene.GetCurrent().properties['indigo'] = d

# load settings
readWriteSettings(0)


		
######  Draw Camera  ###############################################
def drawCamera(y):
####################################################################
	global evtchangesize, Shutter, Autoexp, AutoFocus, evtFocus, HStop, evtHStop
	global FStop, FocusDistance, GroundPlane, SepMatFile, FilmWidth, SizeX, SizeY, ScaleSize, strScaleSize
	global strWhiteBalance, WhiteBalance, whiteBalanceV, ExecuteIndigo, FilmIso
	global ApertureDiffr, AperturePost, ApertureType, BladesNum, BladeOffset, BladeCurvature, StartAngle, ObstacleMap
   
	drawButtons()

	dy=150;

	BGL.glColor3f(1.0,0.5,0.4)
	BGL.glRectf(20,242+y,100,244+y)
	BGL.glColor3f(0.9,0.9,0.9)
	
#  shutter, fstop , iso, and new glare menus by BF
	BGL.glColor3f(0.9,0.9,0.9) ; BGL.glRasterPos2i(10,dy+210) ; Draw.Text("Film ISO:")
	FilmIso = Draw.Menu("Film Iso:%t|25%x25|50%x50|100%x100|200%x200|400%x400|800%x800|1600%x1600", evtNoEvt, 90, dy+205, 75, 18, FilmIso.val, " \"film\" speed")
	BGL.glColor3f(0.9,0.9,0.9) ; BGL.glRasterPos2i(165,dy+230) ; Draw.Text("Film width:")
	FilmWidth = Draw.Number("", evtNoEvt,165, dy+205, 60, 18, FilmWidth.val, 8, 254, "width of the \"film\" in mm")

	BGL.glColor3f(0.9,0.9,0.9) ; BGL.glRasterPos2i(10,dy+190) ; Draw.Text("Exposure:   1/")
	Shutter = Draw.Menu("Exposure duration:%t|1%x1|2%x2|4%x4|8%x8|15%x15|30%x30|60%x60|125%x125|250%x250|500%x500|1000%x1000|2000%x2000", evtNoEvt, 90, dy+185, 75, 18, Shutter.val, "exposure time in secunds") 
	Autoexp = Draw.Toggle("Autoexp", evtNoEvt,165, dy+185, 60, 18, Autoexp.val, "enable auto exposure")
	BGL.glColor3f(0.9,0.9,0.9) ; BGL.glRasterPos2i(10,dy+170) ; Draw.Text("F-Stop:")
	HStop = Draw.Toggle("halfstops", evtNoEvt,165, dy+165, 60, 18,HStop.val, "enable the halfstop range in menu")
	if HStop.val == 0:
		FStop = Draw.Menu("F-Stop:%t|1.4%x14|2%x20|2.8%x28|4%x40|5.6%x56|8%x80|11%x110|16%x160|22%x220|32%x320|64%x640", evtNoEvt, 90, dy+165, 75, 18, FStop.val, "depth of field, (blurring of fore and background)")
	else:
		FStop = Draw.Menu("F-Stop%t|1.2%x12|1.7%x17|2.4%x24|3.3%x33|4.8%x48|6.6%x66|9.6%x96|13.2%x132|19.2%x192|26.4%x264|44%x440", evtNoEvt, 80, dy+165, 75, 18, FStop.val, "depth of field, (blurring of fore and background)")
	BGL.glRasterPos2i(10,150) ; Draw.Text("White bal:")
	WhiteBalance = Draw.Menu(strWhiteBalance, evtNoEvt, 80, 145, 75, 18, WhiteBalance.val, "set the white balance for different lighting conditions")
	FocusDistance = Draw.Number("", evtNoEvt, 80, 125, 75, 18, FocusDistance.val, 0.0, 100, "the distance to objects that are clearly in focus")
	BGL.glRasterPos2i(10,130) ; Draw.Text("Focus dist:")
	Draw.Button("Selected", evtFocusS, 215,135, 60, 18, "get the distance from the selected object")
	Draw.Button("3Dcursor", evtFocusC, 215,115, 60, 18, "get the distance from the 3d cursor")
	AutoFocus = Draw.Toggle("Autofoc", evtNoEvt, 155, 125 , 60, 18, AutoFocus.val, "enable autofocus")
	
	BGL.glColor3f(0.9,0.9,0.9) ; BGL.glRasterPos2i(10,70) ; Draw.Text("Image size:")
	SizeX = Draw.Number("X: ", evtchangesize, 80, 65, 80, 18, SizeX.val, 1, 10000, "width of the render max 10,000")
	SizeY = Draw.Number("Y: ", evtchangesize, 160, 65,80, 18, SizeY.val, 1, 10000, "height of the render max 10,000")
	ScaleSize = Draw.Menu(strScaleSize, evtNoEvt, 240, 65, 80, 18, ScaleSize.val, "scale image size of ...")
	
	ApertureDiffr=Draw.Toggle("Aperture diffraction",evtNoEvt, 230, dy+185, 120, 18, ApertureDiffr.val, "enable aperture diffraction") #will be written in render settings <aperture_diffraction>true/false</aperture_diffraction>
	if ApertureDiffr.val:
		AperturePost=Draw.Toggle("Post Process",evtNoEvt, 350, dy+185, 80, 18, AperturePost.val,"enable Aperture post processing")
	ApertureType = Draw.Menu("Circular%x1|Generated%x2|Image%x3",evtNoEvt,230,dy+165, 120, 18, ApertureType.val, "Type pf aperture")
	if ApertureType == 2:
		BladesNum = Draw.Number("Blades: ",evtNoEvt, 230, dy+145, 100, 18, BladesNum.val, 3, 20, "number of blades")
		BladeOffset = Draw.Number("Offset: ",evtNoEvt, 230, dy+125, 100, 18, BladeOffset.val, 0, 0.5, "blades offset")
		BladeCurvature = Draw.Number("Curvature: ",evtNoEvt, 230, dy+105, 100, 18, BladeCurvature.val, BladeOffset.val, 10, "blade curvature radius")
		StartAngle = Draw.Number("Start Angle: ",evtNoEvt, 230, dy+85, 120, 18, StartAngle.val, 0, 2, "blades start angle")

		BGL.glColor3f(0.9,0.9,0.9) ; BGL.glRasterPos2i(10,dy+70) ; Draw.Text("Obstacle map:")
		ObstacleMap = Draw.String("filename: ", evtNoEvt, 95, dy+65, 300, 18, ObstacleMap.val, 255, "the file name of an .float/.exr probe")
		Draw.Button("....", evtloadobstacle, 395, dy+65, 20 ,18,"open file selector")
	#Bloom = Draw.Toggle("Bloom", evtNoEvt, 310, 135 , 60, 18, Bloom.val, "enable a bloom effect")
	#if Bloom.val==1:
  #	BloomW = Draw.Number("Weight: ", evtNoEvt, 310, 115, 120, 18, BloomW.val, 0.0, 1.0, "bloom intensity")
	#	BloomR = Draw.Number("Radius: ", evtNoEvt, 310, 95, 120, 18, BloomR.val, 0.0, 1.0, "bloom extent")
	#Glare = Draw.Toggle("Glare", evtNoEvt, 310, 215, 60, 18, Glare.val, "enable a glare effect")  #BF
	#if Glare.val == 1:
  #	GlareW =  Draw.Number("Weight: ", evtNoEvt, 310, 175, 120, 18, GlareW.val, 0.0, 1.0, "glare intensity")
  #	GlareR =  Draw.Number("Radius: ", evtNoEvt, 310, 155, 120, 18, GlareR.val, 0.0, 1.0, "glare extent")
	#	GlareB =  Draw.Number("Blades: ", evtNoEvt, 310, 195, 120, 18, GlareB.val, 5, 15, "number of blades")
		
##############  Draw Environment  #######################################
def drawEnv(y):
####################################################
	global evtNoEvt
	global strEnvType, EnvType, EnvFile, strEnvMapType, EnvMapType, EnvGain, EnvWidth, Turbidity, ExtraAtm, GroundPlane, BGColor, BGGain, WorldScale, WorldScaleExp#, SkyGain
	
	drawButtons()
	
	BGL.glColor3f(1.0,0.5,0.4)
	BGL.glRectf(100,242+y,180,244+y)
	BGL.glColor3f(0.9,0.9,0.9)
	
	EnvType = Draw.Menu(strEnvType, evtNoEvt, 10, 205, 205, 18, EnvType.val, "set the environment type")
	if EnvType.val == 2:
		BGL.glColor3f(1.0,1.0,1.0); BGL.glRasterPos2i(10,190); Draw.Text("EXR map:")
		EnvFile = Draw.String("filename: ", evtNoEvt, 80, 185, 330, 18, EnvFile.val, 255, "the file name of an .float/.exr probe")
		EnvMapType = Draw.Menu(strEnvMapType, evtNoEvt, 115, 165, 100, 18, EnvMapType.val, "set the map type of the probe")
		EnvGain = Draw.Number("Gain: ", evtNoEvt, 10, 165, 105, 18, EnvGain.val, 0.001, 1000.00, "gain adjustment, range 0.001-1000")
		Draw.Button("...", evtloadimg, 410, 185, 20 ,18,"open file selector")
		if EnvMapType.val == 0:
			EnvWidth = Draw.Number("Width: ", evtNoEvt, 115, 145, 100, 18, EnvWidth.val, 1, 10000, "input width (= height) of map")
	if EnvType.val == 1:
		Turbidity = Draw.Number("Sky turbidity:", evtNoEvt, 10, 185, 205, 18, Turbidity.val, 1.5, 7.5, "set sky turbidity, range 2 clear - 5 hazy")
		ExtraAtm = Draw.Toggle("Extra Atmospheric",evtNoEvt,10,165,205,18,ExtraAtm.val,"Extra Atmospheric Scattering")
		#SkyGain = Draw.Number("Sky gain:", evtNoEvt, 10, 165, 205, 18, SkyGain.val, 0.0, 5.0, "set sky gain, range 0-5")
	if EnvType.val == 0:
		BGColor = Draw.ColorPicker(evtNoEvt, 10, 145, 30, 60, BGColor.val, "colour and picker (LMB)")
		BGGain = Draw.Number("Gain:", evtNoEvt, 40, 165, 100, 18, BGGain.val, 1e-4, 1e9, "set background color gain")
	GroundPlane = Draw.Toggle("Ground Plane", evtNoEvt, 10, 105, 205, 18, GroundPlane.val, "enable an infinite ground plane at 0,0,0 (not for bidir)")
	BGL.glRasterPos2i(225,128); Draw.Text("Worldscale:","normal")
	WorldScale = Draw.Number("", evtNoEvt, 225, 105, 70, 18, WorldScale.val, 0,9, "scale factor of scene at export else 1 Bl unit == 1 metre in Indigo")  #BF
	WorldScaleExp = Draw.Number("*10^", evtNoEvt, 295, 105, 70, 18, WorldScaleExp.val, -4,6, "Exponent")

###############  Draw Rendersettings   ###################################
def drawSettings(y):
#####################################################
	global MaxDepth, Bidirectional # RRLP StrataWidth
	global AlphaMask, SplatFilter, SplatF_MN_Blur, SplatF_MN_Ring, DownsizeFilter,DownsizeF_MN_Blur, DownsizeF_MN_Ring, MLT, LMP, MaxChange, MaxNumConsRej, SuperSample, BIH, RayNudge, Hybrid
	
	drawButtons()

	
	BGL.glColor3f(1.0,0.5,0.4)
	BGL.glRectf(180,242+y,260,244+y)
	BGL.glColor3f(0.9,0.9,0.9)
	
	
	AlphaMask=Draw.Toggle("AlphaMask",evtNoEvt,10, 310, 80, 18, AlphaMask.val, "enable Alpha Mask rendering")
	BGL.glRasterPos2i(100,335) ; Draw.Text("Splat Filter:", "small")
	SplatFilter=Draw.Menu("Splat Filer:%t | Box %x0 | Gaussian %x1 | Mitchell-Netravali %x2",evtNoEvt,100,310,130,18,SplatFilter.val, "Splat Filter Type")
	if SplatFilter.val == 2:
		SplatF_MN_Blur=Draw.Number("Blur: ",evtNoEvt, 100,280,100,18,SplatF_MN_Blur.val, 0.0, 1.0, "Blur value")
		SplatF_MN_Ring=Draw.Number("Ring: ",evtNoEvt, 100,260,100,18,SplatF_MN_Ring.val, 0.0, 1.0, "Ring value")
	BGL.glRasterPos2i(240,335) ; Draw.Text("Downsize Filter:", "small")
	DownSizeFilter=Draw.Menu("Downsize Filer:%t | Box %x0 | Gaussian %x1 | Mitchell-Netravali %x2",evtNoEvt,240,310,130,18,DownsizeFilter.val, "Downsize Filter Type")
	if DownsizeFilter.val == 2:
		DownsizeF_MN_Blur=Draw.Number("Blur: ",evtNoEvt, 240,280,100,18,DownsizeF_MN_Blur.val, 0.0, 1.0, "Blur value")
		DownsizeF_MN_Ring=Draw.Number("Ring: ",evtNoEvt, 240,260,100,18,DownsizeF_MN_Ring.val, 0.0, 1.0, "Ring value")

	BGL.glRasterPos2i(10,210) ; Draw.Text("Metropolis light transport settings:", "small")
	MLT = Draw.Toggle("Metropolis", evtNoEvt, 10, 125, 80, 18, MLT.val, "enable MLT (useful for indirect light) else pathtracer (for direct light)")
	LMP = Draw.Number("LM probability: ", evtNoEvt, 10, 185, 205, 18, LMP.val, 0.0, 1.0, "the probability of using fresh random numbers")
	MaxChange = Draw.Number("Max Change: ", evtNoEvt, 10, 165, 205, 18, MaxChange.val, 0.0, 1.0, "the maximum mutation size") 
	MaxNumConsRej = Draw.Number("Max num consec reject:", evtNoEvt, 10, 145, 205, 18, MaxNumConsRej.val, 0, 10000, "low values give more biased calculations and fewer fireflies")
	
	SuperSample = Draw.Number("Supersamples:", evtNoEvt, 225,165,205,18, SuperSample.val,1,4, "more helps anti alias edges but eats memory") #BF
	
	BGL.glRasterPos2i(225,210) ; Draw.Text("General tracing parameters:", "small")
	Bidirectional = Draw.Toggle("Bidirectional", evtNoEvt, 225, 125, 80, 18, Bidirectional.val, "enable bidirectional tracing (useful for indirect light and caustics)")
	Hybrid = Draw.Toggle("Hybrid", evtNoEvt, 75, 105, 80, 18, Hybrid.val, "enable hybrid pathtracing/MLT mode (Metro setting is ignored)") 
#BF	RRLP = Draw.Number("RR live probability: ", evtNoEvt, 225, 185, 205, 18, RRLP.val, 0.0, 1.0, "Russian roulette live probability")
	MaxDepth = Draw.Number("Max depth:", evtNoEvt, 225, 185, 205, 18, MaxDepth.val, 1, 10000, "maximum ray bounce depth, range 1-10000")
	BIH = Draw.Number("hybrid threshold: ", evtNoEvt, 10, 85, 205, 18, BIH.val, 1, 10000000, "the faces threshold for using hybid mode")	#BF
	RayNudge = Draw.Number("ray nudge: ",evtNoEvt, 225, 85, 205, 18, RayNudge.val,0.00001, 1,"ray origin nudge distance")
	#BGL.glRasterPos2i(10,175) ; Draw.Text("Path tracer settings")
#BF	StrataWidth = Draw.Number("Strata width:", evtNoEvt, 225, 145, 205, 18, StrataWidth.val, 1, 50, "number of samples per pixel, range 1-50")
	

##################  Draw RSettings  #########################   
def drawSystem(y):
########################################
	global Logging
	global SaveUTMExr, SaveTMExr, SaveIgi, UseGUI, Watermark, DisplayPeriod, ColExponent
	global HaltTime, HaltSamples, FrameUp, ImageSave, SepMatFile, Threads, CacheTree
	
	drawButtons()
	
	BGL.glColor3f(1.0,0.5,0.4)
	BGL.glRectf(340,242+y,420,244+y)
	BGL.glColor3f(0.9,0.9,0.9)

	
	
	CacheTree =  Draw.Toggle("Cache to disk", evtNoEvt, 225, 205, 80, 18, CacheTree.val, "enable caching KD-trees to disk") #BF
	Logging = Draw.Toggle("Logging", evtNoEvt, 340, 205, 80, 18, Logging.val, "enable writing a log.txt")
	BGL.glRasterPos2i(225,190) ; Draw.Text("Image saving options:", "small")
	SaveUTMExr = Draw.Toggle("Save utm EXR", evtNoEvt, 225, 165, 205, 18, SaveUTMExr.val, "save image also as an untonemapped .exr file")
	SaveTMExr = Draw.Toggle("Save tm EXR", evtNoEvt, 225, 145, 205, 18, SaveTMExr.val, "save image also as a tonemapped .exr file")
	SaveIgi = Draw.Toggle("Save IGI", evtNoEvt, 225, 125, 205, 18, SaveIgi.val, "save image also as an .igi file WARNING: can be large and slow")
	UseGUI = Draw.Toggle("Use GUI",evtNoEvt, 225, 105, 205, 18, UseGUI.val,"Use Indigo GUI")

	Watermark = Draw.Toggle("Watermark",evtNoEvt, 125, 80, 205, 18, Watermark.val,"Add watermark logo")
	
	ColExponent = Draw.Number("Gamma: ", evtNoEvt, 10, 225, 205, 18, ColExponent.val, 0.01, 5.0, "overall gamma correction")
	Threads = Draw.Number("Threads:", evtNoEvt, 10,205,205,18, Threads.val,1,4, "the number of processor threads/cores to be used")
	FrameUp = Draw.Number("Frame Upl:", evtNoEvt, 10, 185, 205, 18, FrameUp.val, 20, 10000, "the time between network slave uploads of image to master [sec]")
	HaltTime = Draw.Number("Halt time:", evtNoEvt, 10, 165, 205, 18, HaltTime.val, -1, 86400, "the rendering duration [sec] setting -1 will continue until manual exit")
	HaltSamples = Draw.Number("Halt Samples per Pixel:", evtNoEvt, 10, 145, 205, 18, HaltSamples.val, -1, 10000, "the rendering quality setting -1 will continue until manual exit")
	DisplayPeriod = Draw.Number("Display period:", evtNoEvt, 10, 125, 205, 18, DisplayPeriod.val, 10, 10000, "the time between GUI display updates [sec]")
	ImageSave = Draw.Number("Save period:", evtNoEvt, 10, 105, 205, 18, ImageSave.val, 20, 10000, "the time between saving .png images [sec]")
	
	
	
#################  Draw Tonemapping  #########################
def drawTonemap(y):
#######################################

	global ToneMapType, ToneMapPreScale, ToneMapPostScale, ToneMapBurn, ToneMapScale, ToneMapScaleExp, ToneMapEV
	
	drawButtons()
	
	BGL.glColor3f(1.0,0.5,0.4)
	BGL.glRectf(260,242+y,340,244+y)
	BGL.glColor3f(0.9,0.9,0.9)
	
	ToneMapType = Draw.Menu(strToneMapType, evtNoEvt, 10, 205, 105, 18, ToneMapType.val, "set the type of the tonemapping")
	
	if ToneMapType.val == 0:
		ToneMapScale = Draw.Number("Scale: ", evtNoEvt, 10, 185, 105, 18, ToneMapScale.val, 1, 9, "Linear scale: refer to the Indigo Manual ;)")
		ToneMapScaleExp = Draw.Number("*10^: ", evtNoEvt, 115, 185, 100, 18, ToneMapScaleExp.val, -20, 20, "Linear scale exponent")
	if ToneMapType.val == 1:
		ToneMapBurn = Draw.Number("Burn: ", evtNoEvt, 10, 185, 105, 18, ToneMapBurn.val, 0.1, 10.0, "burn to white, 0.1= a lot, 10= none")
		ToneMapPreScale = Draw.Number("PreS: ", evtNoEvt, 115, 185, 100, 18, ToneMapPreScale.val, 0,50.0, "scales the mid-grey-point of the rendered pixels, practical range 1-10")
		ToneMapPostScale = Draw.Number("PostS: ", evtNoEvt, 215, 185, 105, 18, ToneMapPostScale.val, 0,50.0, "scales the white-point of the rendered pixels , practical range 1-10 ")
	if ToneMapType.val == 2:
		ToneMapEV=Draw.Number("EV Adjust: ",evtNoEvt,10,185,105,18,ToneMapEV.val,-3,3,"Exposure Value Adjustment")
	
	#
#   DiffuseGain = Draw.Number("Diffuse gain: ", evtNoEvt, 130, 105, 150, 18, DiffuseGain.val, 0.0, 1.0, "overall diffuse-matt color gain")
#   SpecularGain = Draw.Number("Specular gain: ", evtNoEvt, 300, 105, 150, 18, SpecularGain.val, 0.0, 1.0, "overall specular-transparent color gain")
	
##############
def drawGUI():
##############

	global Screen
	
	if Screen==0:
		drawCamera(230)
		#drawCamera(130)
	if Screen==1:
		drawEnv(230)
	if Screen==2:
		drawSettings(230)
	if Screen==3:
		drawSystem(230)
	if Screen==4:
		drawTonemap(230)
	if Screen==5:
		materialGUI(0) # zuegs: added material editor
	#Draw.Redraw(1)

def drawScreenButtons(y):
	global openCamera, openEnv, openRSet, openSSet, openTmap, matGui
	BGL.glColor3f(0.2,0.2,0.2)
	BGL.glRectf(0,0,440,294+y)
	BGL.glColor3f(0.9,0.9,0.9);BGL.glRasterPos2i(10,275+y) ; Draw.Text("B L E N D I G O  v1.0.8  ::  Indigo exporter module", "small")
	BGL.glColor3f(0.9, 0.9, 0.9) ; BGL.glRasterPos2i(340,275+y) ; Draw.Text("Press Q or ESC to quit", "tiny")
	
	Draw.Button("Camera", openCamera, 20, 245+y, 80, 18, "set camera parameters")
	Draw.Button("Environment", openEnv, 100, 245+y, 80, 18, "set environment parameters")
	Draw.Button("Renderer", openRSet, 180, 245+y, 80, 18, "set rendering parameters")
	Draw.Button("Tonemap", openTmap, 260, 245+y, 80, 18, "set tonemap parameters")
	Draw.Button("System", openSSet, 340, 245+y, 80, 18, "set system options")
	matGui=Draw.Toggle("Materials", openMat, 20, 223+y, 80, 18, matGui.val, "show material editor")

def drawExportButtons(y):
	global envExport, envExportAmin, envNoEnv, ExecuteIndigo, DefaultExport, ExportMeshs, SepMatFile, SepMeshFile
	Draw.Button("EXPORT Scene", evtExport, 50, 30, 160, 24, "open file selector to export and save Indigo .igs")
	Draw.Button("EXPORT Animation", evtExportAnim, 230, 30, 160, 24, "open file selector to export and save for each frame WARNING: eats disk space")
	ExecuteIndigo = Draw.Toggle("Autorun", evtNoEvt, 20,5,80,18, ExecuteIndigo.val, "enable auto start of Indigo after export - to path set in Indigowrapper.conf")
	DefaultExport = Draw.Toggle("default.igs",evtNoEvt,100,5,80,18, DefaultExport.val, "use \"default.igs\" as the file name")
	ExportMeshs = Draw.Toggle("exp. meshes",evtNoEvt,180,5,80,18, ExportMeshs.val, "enable export of meshes")
	if not ExportMeshs.val: SepMeshFile.val = 1  # without mesh export force use of external mesh file
	SepMeshFile = Draw.Toggle("sep. meshes",evtNoEvt,260,5,80,18, SepMeshFile.val, "enable saving meshes to a separate file")
	SepMatFile = Draw.Toggle("sep. mats", evtNoEvt, 340,5,80,18, SepMatFile.val, "enable saving materials to a separate file")

##################
def drawButtons():
##################
	## buttons
	drawScreenButtons(230)
	drawExportButtons(0)

def event(evt, val):  # function that handles keyboard and mouse events
	if evt == Draw.ESCKEY or evt == Draw.QKEY:
		stop = Draw.PupMenu("OK?%t|cancel export %x1")
		if stop == 1:
			Draw.Exit()
			return
	materialEvent(evt, val) # zuegs: added material editor


def setEnvMap(efilename):
#	EnvFile.val = os.path.basename(efilename)
	EnvFile.val = efilename

def setObstacleMap(filename):
	ObstacleMap.val = filename
	
def buttonEvt(evt):  # function that handles button events	
	global Screen, EnFile, indigopath, lastScreen


	if evt:
		Draw.Redraw(1)

	if evt == evtExport:
		readWriteSettings(1)
		if DefaultExport.val == 1:
#			datadir=Blender.Get("datadir")
#			f = open(os.path.join(datadir + os.sep + 'IndigoWrapper.conf'), 'r')
#			ic = splitn(f.readline())
#			f.close()
			filename = os.path.join(indigopath, "default.igs")
			exportStill(filename)
		else:
#			datadir=Blender.Get("datadir")
#			f = open(os.path.join(datadir + os.sep + 'IndigoWrapper.conf'), 'r')
#			ic = splitn(f.readline())
#			f.close()
			Blender.Window.FileSelector(exportStill, "export", newFName('igs'))
	
	if evt == evtExportAnim:
		#Switch sep.meshes and sep.mats to on for animation
		#SepMeshFile.val=1
		SepMatFile.val=1
		ExportMeshs.val=1
		Blender.Window.FileSelector(exportAnim, "export animation", newFName('igs'))
		   		
	if evt == evtNoEvt:
		Draw.Redraw(1)
		readWriteSettings(1)
		
	if evt == evtchangesize:
		Scene.GetCurrent().getRenderingContext().imageSizeX(SizeX.val);
		Scene.GetCurrent().getRenderingContext().imageSizeY(SizeY.val);
		Draw.Redraw(1)
		
	if evt == openCamera:
		matGui.val=0
		Screen = 0
		drawGUI()
		Draw.Redraw(1)
	
	if evt == openEnv:
		matGui.val=0
		Screen = 1
		drawGUI()
		Draw.Redraw(1)
		
	if evt == openRSet:
		matGui.val=0
		Screen = 2
		drawGUI()
		Draw.Redraw(1)
	
	if evt == openSSet:
		matGui.val=0
		Screen = 3
		drawGUI()
		Draw.Redraw(1)
	
	if evt == openTmap:
		matGui.val=0
		Screen = 4
		drawGUI()
		Draw.Redraw(1)
	
	if evt == evtloadimg:
		Blender.Window.FileSelector(setEnvMap, "load environment map")
		#Draw.Redraw(1)
		readWriteSettings(1)

	if evt == evtloadobstacle:
		Blender.Window.FileSelector(setObstacleMap,"load obstacle map")
	
	if evt == evtFocusS:
		setFocus("S")
		#Draw.Redraw(1)
		readWriteSettings(1)

	if evt == evtFocusC:
		setFocus("C")
		#Draw.Redraw(1)
		readWriteSettings(1)

	if evt == openMat:
		if matGui.val:
			lastScreen = Screen
			Screen = 5
			drawGUI()
			Draw.Redraw(1)
			#Draw.Draw()
		else:
			Screen = lastScreen
			drawGUI()
			Draw.Redraw(1)
			#Draw.Draw()

	if evt == evtMatLightIES:
		if matLightIES == 1:
			matLightEfficacyScale.val=0

	if evt == evtMatLightEfficacy:
		if matLightEfficacyScale == 1:
			matLightIES.val=0

	materialButtonEvt(evt) # zuegs: added material editor
	Draw.Redraw(1)


def setFocus(target):
	global FocusDistance
	currentscene = Scene.GetCurrent()
	camObj = currentscene.getCurrentCamera()
	loc1 = camObj.getLocation()
	if target == "S":
		
		try:	
			selObj = Object.GetSelected()[0]
			loc2 = selObj.getLocation()
		except:
			print "select an object to focus on\n"
			#OK?%t|cancel export %x1
			Draw.PupMenu("Error%t|Select an Object")
			return
	if target == "C":
		loc2 = Window.GetCursorPos()
	ws=WorldScale.val*(10**WorldScaleExp.val)
	FocusDistance.val = ws *((((loc1[0]-loc2[0])**2)+((loc1[1]-loc2[1])**2)+((loc1[2]-loc2[2])**2))**0.5)

Screen = 0
lastScreen = 0
Draw.Register(drawGUI, event, buttonEvt)
